@@ -30,7 +30,7 @@ std::list<CControlSocket::t_lockInfo> CControlSocket::m_lockInfoList;
 BEGIN_EVENT_TABLE(CControlSocket, wxEvtHandler)
 	EVT_TIMER(wxID_ANY, CControlSocket::OnTimer)
 	EVT_COMMAND(wxID_ANY, fzOBTAINLOCK, CControlSocket::OnObtainLock)
-END_EVENT_TABLE();
+END_EVENT_TABLE()
 
 COpData::COpData(enum Command op_Id)
 	: opId(op_Id)
@@ -103,7 +103,7 @@ void CControlSocket::LogTransferResultMessage(int nErrorCode, CFileTransferOpDat
 		wxString time = wxString::Format(
 			wxPLURAL("%d second", "%d seconds", elapsed),
 			elapsed);
-		
+
 		wxLongLong transferred = m_pTransferStatus->currentOffset - m_pTransferStatus->startOffset;
 		wxString size = CSizeFormatBase::Format(m_pEngine->GetOptions(), transferred, true);
 
@@ -405,16 +405,19 @@ bool CControlSocket::ParsePwdReply(wxString reply, bool unquoted /*=false*/, con
 				reply = _T("");
 		}
 		else
+		{
 			reply = reply.Mid(pos1 + 1, pos2 - pos1 - 1);
+			reply.Replace(_T("\"\""), _T("\""));
+		}
 	}
 
 	m_CurrentPath.SetType(m_pCurrentServer->GetType());
 	if (reply == _T("") || !m_CurrentPath.SetPath(reply))
 	{
 		if (reply != _T(""))
-			LogMessage(__TFILE__, __LINE__, this, Debug_Warning, _T("Failed to parse returned path."));
+			LogMessage(::Error, _("Failed to parse returned path."));
 		else
-			LogMessage(__TFILE__, __LINE__, this, Debug_Warning, _T("Server returned empty path."));
+			LogMessage(::Error, _("Server returned empty path."));
 
 		if (!defaultPath.IsEmpty())
 		{
@@ -483,15 +486,7 @@ int CControlSocket::CheckOverwriteFile()
 	else
 		pNotification->canResume = false;
 
-	wxStructStat buf;
-	int result;
-	result = wxStat(pData->localFile, &buf);
-	if (!result)
-	{
-		pNotification->localTime = wxDateTime(buf.st_mtime);
-		if (!pNotification->localTime.IsValid())
-			pNotification->localTime = wxDateTime(buf.st_ctime);
-	}
+	pNotification->localTime = CLocalFileSystem::GetModificationTime(pData->localFile);
 
 	if (pData->fileTime.IsValid())
 		pNotification->remoteTime = pData->fileTime;
@@ -515,7 +510,7 @@ int CControlSocket::CheckOverwriteFile()
 
 CFileTransferOpData::CFileTransferOpData(bool is_download, const wxString& local_file, const wxString& remote_file, const CServerPath& remote_path) :
 	COpData(cmd_transfer),
- 	localFile(local_file), remoteFile(remote_file), remotePath(remote_path),
+	localFile(local_file), remoteFile(remote_file), remotePath(remote_path),
 	download(is_download),
 	localFileSize(-1), remoteFileSize(-1),
 	tryAbsolutePath(false), resume(false), transferInitiated(false)
@@ -573,21 +568,7 @@ wxChar* CControlSocket::ConvToLocalBuffer(const char* buffer, wxMBConv& conv)
 
 	wchar_t* unicode = new wchar_t[len + 1];
 	conv.MB2WC(unicode, buffer, len + 1);
-#if wxUSE_UNICODE
 	return unicode;
-#else
-	len = wxConvCurrent->WC2MB(0, unicode, 0);
-	if (!len)
-	{
-		delete [] unicode;
-		return 0;
-	}
-
-	wxChar* output = new wxChar[len + 1];
-	wxConvCurrent->WC2MB(output, unicode, len + 1);
-	delete [] unicode;
-	return output;
-#endif
 }
 
 wxChar* CControlSocket::ConvToLocalBuffer(const char* buffer)
@@ -614,13 +595,7 @@ wxChar* CControlSocket::ConvToLocalBuffer(const char* buffer)
 	}
 
 	// Fallback: Conversion using current locale
-#if wxUSE_UNICODE
 	wxChar* res = ConvToLocalBuffer(buffer, *wxConvCurrent);
-#else
-	// No conversion needed, just copy
-	wxChar* res = new wxChar[strlen(buffer) + 1];
-	strcpy(res, buffer);
-#endif
 
 	return res;
 }
@@ -629,30 +604,14 @@ wxCharBuffer CControlSocket::ConvToServer(const wxString& str, bool force_utf8 /
 {
 	if (m_useUTF8 || force_utf8)
 	{
-#if wxUSE_UNICODE
 		wxCharBuffer buffer = wxConvUTF8.cWX2MB(str);
-#else
-		wxWCharBuffer unicode = wxConvCurrent->cMB2WC(str);
-		wxCharBuffer buffer;
-		if (unicode)
-			 buffer = wxConvUTF8.cWC2MB(unicode);
-#endif
-
 		if (buffer || force_utf8)
 			return buffer;
 	}
 
 	if (m_pCSConv)
 	{
-#if wxUSE_UNICODE
 		wxCharBuffer buffer = m_pCSConv->cWX2MB(str);
-#else
-		wxWCharBuffer unicode = wxConvCurrent->cMB2WC(str);
-		wxCharBuffer buffer;
-		if (unicode)
-			 buffer = m_pCSConv->cWC2MB(unicode);
-#endif
-
 		if (buffer)
 			return buffer;
 	}
@@ -757,7 +716,7 @@ bool CControlSocket::TryLockCache(enum locking_reason reason, const CServerPath&
 		wxASSERT(own->reason == reason);
 	}
 
-	// Needs to be set in any case so that ResetOperation 
+	// Needs to be set in any case so that ResetOperation
 	// unlocks or cancels the lock wait
 	m_pCurOpData->holdsLock = true;
 
@@ -1031,17 +990,17 @@ void CRealControlSocket::OnSocketEvent(CSocketEvent &event)
 	case CSocketEvent::hostaddress:
 		{
 			const wxString& address = event.GetData();
-			LogMessage(Status, _("Connecting to %s..."), address.c_str()); 
+			LogMessage(Status, _("Connecting to %s..."), address.c_str());
 		}
 		break;
 	case CSocketEvent::connection_next:
 		if (event.GetError())
-			LogMessage(Status, _("Connection attempt failed with \"%s\", trying next address."), CSocket::GetErrorDescription(event.GetError()).c_str()); 
+			LogMessage(Status, _("Connection attempt failed with \"%s\", trying next address."), CSocket::GetErrorDescription(event.GetError()).c_str());
 		break;
 	case CSocketEvent::connection:
 		if (event.GetError())
 		{
-			LogMessage(Status, _("Connection attempt failed with \"%s\"."), CSocket::GetErrorDescription(event.GetError()).c_str()); 
+			LogMessage(Status, _("Connection attempt failed with \"%s\"."), CSocket::GetErrorDescription(event.GetError()).c_str());
 			OnClose(event.GetError());
 		}
 		else
@@ -1141,7 +1100,10 @@ int CRealControlSocket::Connect(const CServer &server)
 	SetWait(true);
 
 	if (server.GetEncodingType() == ENCODING_CUSTOM)
+	{
+		LogMessage(Debug_Info, _T("Using custom encoding: %s"), server.GetCustomEncoding().c_str());
 		m_pCSConv = new wxCSConv(server.GetCustomEncoding());
+	}
 
 	delete m_pCurrentServer;
 	m_pCurrentServer = new CServer(server);
@@ -1438,3 +1400,45 @@ void CControlSocket::CreateLocalDir(const wxString &local_file)
 	n->dir = last_successful;
 	m_pEngine->AddNotification(n);
 }
+
+int CControlSocket::List(CServerPath, wxString, int)
+{
+	return FZ_REPLY_NOTSUPPORTED;
+}
+
+int CControlSocket::FileTransfer(const wxString, const CServerPath &,
+					const wxString &, bool,
+					const CFileTransferCommand::t_transferSettings&)
+{
+	return FZ_REPLY_NOTSUPPORTED;
+}
+
+int CControlSocket::RawCommand(const wxString&)
+{
+	return FZ_REPLY_NOTSUPPORTED;
+}
+
+int CControlSocket::Delete(const CServerPath&, const std::list<wxString>&)
+{
+	return FZ_REPLY_NOTSUPPORTED;
+}
+
+int CControlSocket::RemoveDir(const CServerPath&, const wxString&)
+{
+	return FZ_REPLY_NOTSUPPORTED;
+}
+
+int CControlSocket::Mkdir(const CServerPath&)
+{
+	return FZ_REPLY_NOTSUPPORTED;
+}
+
+int CControlSocket::Rename(const CRenameCommand&)
+{
+	return FZ_REPLY_NOTSUPPORTED;
+}
+
+int CControlSocket::Chmod(const CChmodCommand&)
+{
+	return FZ_REPLY_NOTSUPPORTED;
+}
\ No newline at end of file
