@@ -354,7 +354,7 @@ public:
 					CToken *pToken = new CToken(m_pLine + start, m_parsePos - start);
 					m_Tokens.push_back(pToken);
 
-					while ((m_pLine[m_parsePos] == ' ' || m_pLine[m_parsePos] == '\t') && m_parsePos < m_len)
+					while (m_parsePos < m_len && (m_pLine[m_parsePos] == ' ' || m_pLine[m_parsePos] == '\t'))
 						++m_parsePos;
 
 					if (m_Tokens.size() > n)
@@ -447,14 +447,16 @@ protected:
 	wxChar* m_pLine;
 };
 
-CDirectoryListingParser::CDirectoryListingParser(CControlSocket* pControlSocket, const CServer& server)
-	: m_pControlSocket(pControlSocket), m_server(server)
+CDirectoryListingParser::CDirectoryListingParser(CControlSocket* pControlSocket, const CServer& server, listingEncoding::type encoding)
+	: m_pControlSocket(pControlSocket)
+	, m_currentOffset(0)
+	, m_totalData()
+	, m_prevLine(0)
+	, m_server(server)
+	, m_fileListOnly(true)
+	, m_maybeMultilineVms(false)
+	, m_listingEncoding(encoding)
 {
-	m_currentOffset = 0;
-	m_prevLine = 0;
-	m_fileListOnly = true;
-	m_maybeMultilineVms = false;
-
 	if (m_MonthNamesMap.empty())
 	{
 		//Fill the month names map
@@ -514,7 +516,7 @@ CDirectoryListingParser::CDirectoryListingParser(CControlSocket* pControlSocket,
 
 		//French month names
 		m_MonthNamesMap[_T("janv")] = 1;
-		m_MonthNamesMap[_T("f\xe9")_T("b")] = 1;
+		m_MonthNamesMap[_T("f\xe9") _T("b")] = 1;
 		m_MonthNamesMap[_T("f\xe9v")] = 2;
 		m_MonthNamesMap[_T("fev")] = 2;
 		m_MonthNamesMap[_T("f\xe9vr")] = 2;
@@ -528,7 +530,7 @@ CDirectoryListingParser::CDirectoryListingParser(CControlSocket* pControlSocket,
 		m_MonthNamesMap[_T("ao\xfb")] = 8;
 		m_MonthNamesMap[_T("ao\xfbt")] = 8;
 		m_MonthNamesMap[_T("aout")] = 8;
-		m_MonthNamesMap[_T("d\xe9")_T("c")] = 12;
+		m_MonthNamesMap[_T("d\xe9") _T("c")] = 12;
 		m_MonthNamesMap[_T("dec")] = 12;
 
 		//Italian month names
@@ -563,6 +565,10 @@ CDirectoryListingParser::CDirectoryListingParser(CControlSocket* pControlSocket,
 		m_MonthNamesMap[_T("sie")] = 8;
 		m_MonthNamesMap[_T("wrz")] = 9;
 		m_MonthNamesMap[_T("pa\x9f")] = 10;
+		m_MonthNamesMap[_T("pa\xbc")] = 10; // ISO-8859-2 
+		m_MonthNamesMap[_T("paz")] = 10; // ASCII 
+		m_MonthNamesMap[_T("pa\xc5\xba")] = 10; // UTF-8 
+		m_MonthNamesMap[_T("pa\x017a")] = 10; // some servers send this
 		m_MonthNamesMap[_T("lis")] = 11;
 		m_MonthNamesMap[_T("gru")] = 12;
 
@@ -605,13 +611,11 @@ CDirectoryListingParser::CDirectoryListingParser(CControlSocket* pControlSocket,
 		m_MonthNamesMap[_T("avg")] = 8;
 
 		//Icelandic
-#if wxUSE_UNICODE
 		m_MonthNamesMap[_T("ma\x00ed")] = 5;
 		m_MonthNamesMap[_T("j\x00fan")] = 6;
 		m_MonthNamesMap[_T("j\x00fal")] = 7;
 		m_MonthNamesMap[_T("\x00e1g")] = 8;
 		m_MonthNamesMap[_T("n\x00f3v")] = 11;
-#endif
 		m_MonthNamesMap[_T("des")] = 12;
 
 		//Lithuanian
@@ -628,6 +632,9 @@ CDirectoryListingParser::CDirectoryListingParser(CControlSocket* pControlSocket,
 		m_MonthNamesMap[_T("lap")] = 11;
 		m_MonthNamesMap[_T("grd")] = 12;
 
+		// Hungarian
+		m_MonthNamesMap[_T("szept")] = 9;
+
 		//There are more languages and thus month
 		//names, but as long as nobody reports a
 		//problem, I won't add them, there are way
@@ -688,6 +695,8 @@ CDirectoryListingParser::~CDirectoryListingParser()
 
 bool CDirectoryListingParser::ParseData(bool partial)
 {
+	DeduceEncoding();
+
 	bool error = false;
 	CLine *pLine = GetLine(partial, error);
 	while (pLine)
@@ -1193,7 +1202,7 @@ bool CDirectoryListingParser::ParseUnixDateTime(CLine *pLine, int &index, CDiren
 			int fileDayOfYear = (month - 1) * 31 + day;
 
 			// We have to compare with an offset of one. In the worst case,
-			// the server's timezone might be up to 24 hours ahead of the 
+			// the server's timezone might be up to 24 hours ahead of the
 			// client.
 			// Problem: Servers which do send the time but not the year even
 			// one day away from getting 1 year old. This is far more uncommon
@@ -1680,7 +1689,7 @@ bool CDirectoryListingParser::ParseAsVms(CLine *pLine, CDirentry &entry)
 	// Current token is either size or date
 	bool gotSize = false;
 	pos = token.Find('/');
-	
+
 	if (!pos)
 		return false;
 
@@ -1838,7 +1847,7 @@ bool CDirectoryListingParser::ParseAsIbm(CLine *pLine, CDirentry &entry)
 		entry.name.RemoveLast();
 		entry.flags |= CDirentry::flag_dir;
 	}
-	
+
 	if (entry.has_time())
 		entry.time.Add(m_timezoneOffset);
 
@@ -2025,11 +2034,17 @@ bool CDirectoryListingParser::ParseOther(CLine *pLine, CDirentry &entry)
 
 bool CDirectoryListingParser::AddData(char *pData, int len)
 {
+	ConvertEncoding( pData, len );
+
 	t_list item;
 	item.p = pData;
 	item.len = len;
 
 	m_DataList.push_back(item);
+	m_totalData += len;
+
+	if (m_totalData < 512)
+		return true;
 
 	return ParseData(true);
 }
@@ -2132,7 +2147,7 @@ CLine *CDirectoryListingParser::GetLine(bool breakAtEnd /*=false*/, bool &error)
 		char *res = new char[reslen + 1];
 		res[reslen] = 0;
 
-		int	respos = 0;
+		int respos = 0;
 
 		// Copy line data
 		std::list<t_list>::iterator i = m_DataList.begin();
@@ -2534,7 +2549,7 @@ bool CDirectoryListingParser::ParseAsIBM_MVS_Tape(CLine *pLine, CDirentry &entry
 	entry.ownerGroup = _T("");
 	entry.permissions = _T("");
 	entry.size = -1;
-	
+
 	if (pLine->GetToken(index++, token))
 		return false;
 
@@ -2674,17 +2689,27 @@ int CDirectoryListingParser::ParseAsMlsd(CLine *pLine, CDirentry &entry)
 		wxString value = facts.Mid(pos + 1, delim - pos - 1);
 		if (factname == _T("type"))
 		{
-			if (!value.CmpNoCase(_T("dir")))
+			int pos = value.Find(':');
+			wxString valuePrefix;
+			if (pos == -1)
+				valuePrefix = value;
+			else
+				valuePrefix = value.Left(pos);
+			MakeLowerAscii(valuePrefix);
+
+			if (valuePrefix == _T("dir") && pos == -1)
 				entry.flags |= CDirentry::flag_dir;
-			else if (!value.Left(13).CmpNoCase(_T("OS.unix=slink")))
+			else if (valuePrefix == _T("os.unix=slink") || valuePrefix == _T("os.unix=symlink"))
 			{
 				entry.flags |= CDirentry::flag_dir | CDirentry::flag_link;
-				if (value[13] == ':' && value[14] != 0)
-					entry.target = value.Mid(14);
+				if (pos != -1)
+					entry.target = value.Mid(pos);
 			}
-			else if (!value.CmpNoCase(_T("cdir")) ||
-					 !value.CmpNoCase(_T("pdir")))
+			else if ((valuePrefix == _T("cdir") || valuePrefix == _T("pdir")) && pos == -1)
+			{
+				// Current and parent directory, don't parse it
 				return 2;
+			}
 		}
 		else if (factname == _T("size"))
 		{
@@ -3006,8 +3031,99 @@ bool CDirectoryListingParser::GetMonthFromName(const wxString& name, int &month)
 		if (iter == m_MonthNamesMap.end())
 			return false;
 	}
-	
+
 	month = iter->second;
-	
+
 	return true;
 }
+
+char ebcdic_table[256] = {
+	' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  // 0
+	' ',  ' ',  ' ',  ' ',  ' ',  '\n', ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  '\n', // 1
+	' ',  ' ',  ' ',  ' ',  ' ',  '\n', ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  // 2
+	' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  // 3
+	' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  '.',  '<',  '(',  '+',  '|',  // 4
+	'&',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  '!',  '$',  '*',  ')',  ';',  ' ',  // 5
+	'-',  '/',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  '|',  ',',  '%',  '_',  '>',  '?',  // 6
+	' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  '`',  ':',  '#',  '@',  '\'', '=',  '"',  // 7
+	' ',  'a',  'b',  'c',  'd',  'e',  'f',  'g',  'h',  'i',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  // 8
+	' ',  'j',  'k',  'l',  'm',  'n',  'o',  'p',  'q',  'r',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  // 9
+	' ',  '~',  's',  't',  'u',  'v',  'w',  'x',  'y',  'z',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  // a
+	'^',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  '[',  ']',  ' ',  ' ',  ' ',  ' ',  // b
+	'{',  'A',  'B',  'C',  'D',  'E',  'F',  'G',  'H',  'I',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  // c
+	'}',  'J',  'K',  'L',  'M',  'N',  'O',  'P',  'Q',  'R',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  // d
+	'\\', ' ',  'S',  'T',  'U',  'V',  'W',  'X',  'Y',  'Z',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  // e
+	'0',  '1',  '2',  '3',  '4',  '5',  '6',  '7',  '8',  '9',  ' ',  ' ',  ' ',  ' ',  ' ',  ' '   // f
+};
+
+void CDirectoryListingParser::ConvertEncoding(char *pData, int len)
+{
+	if (m_listingEncoding != listingEncoding::ebcdic)
+		return;
+
+	for (int i = 0; i < len; ++i)
+	{
+		pData[i] = ebcdic_table[static_cast<unsigned char>(pData[i])];
+	}
+}
+
+void CDirectoryListingParser::DeduceEncoding()
+{
+	if (m_listingEncoding != listingEncoding::unknown)
+		return;
+
+	int count[256];
+
+	memset(&count, 0, sizeof(int)*256);
+
+	for (std::list<t_list>::const_iterator it = m_DataList.begin(); it != m_DataList.end(); ++it)
+	{
+		for (int i = 0; i < it->len; ++i)
+			++count[static_cast<unsigned char>(it->p[i])];
+	}
+
+	int count_normal = 0;
+	int count_ebcdic = 0;
+	for (int i = '0'; i <= '9'; ++i ) {
+		count_normal += count[i];
+	}
+	for (int i = 'a'; i <= 'Z'; ++i ) {
+		count_normal += count[i];
+	}
+	for (int i = 'A'; i <= 'Z'; ++i ) {
+		count_normal += count[i];
+	}
+
+	for (int i = 0x81; i <= 0x89; ++i ) {
+		count_ebcdic += count[i];
+	}
+	for (int i = 0x91; i <= 0x99; ++i ) {
+		count_ebcdic += count[i];
+	}
+	for (int i = 0xa2; i <= 0xa9; ++i ) {
+		count_ebcdic += count[i];
+	}
+	for (int i = 0xc1; i <= 0x89; ++i ) {
+		count_ebcdic += count[i];
+	}
+	for (int i = 0xd1; i <= 0x99; ++i ) {
+		count_ebcdic += count[i];
+	}
+	for (int i = 0xe2; i <= 0xa9; ++i ) {
+		count_ebcdic += count[i];
+	}
+	for (int i = 0xf0; i <= 0xf9; ++i ) {
+		count_ebcdic += count[i];
+	}
+
+
+	if ((count[0x1f] || count[0x15] || count[0x25]) && !count[0x0a] && count['@'] && count['@'] > count[' '] && count_ebcdic > count_normal)
+	{
+		m_pControlSocket->LogMessage(::Status, _("Received a directory listing which appears to be encoded in EBCDIC."));
+		m_listingEncoding = listingEncoding::ebcdic;
+		for (std::list<t_list>::iterator it = m_DataList.begin(); it != m_DataList.end(); ++it)
+			ConvertEncoding(it->p, it->len);
+	}
+	else
+		m_listingEncoding = listingEncoding::normal;
+}
