@@ -1,5 +1,6 @@
 #include <filezilla.h>
 #include "serverpath.h"
+#include "string_coalescer.h"
 
 #define FTP_MVS_DOUBLE_QUOTE (wxChar)0xDC
 
@@ -17,16 +18,16 @@ struct CServerTypeTraits
 };
 
 static const CServerTypeTraits traits[SERVERTYPE_MAX] = {
-	{ _T("/"),  true,  0,    0,    false, 0, 0,   true,  false }, // Failsafe
-	{ _T("/"),  true,  0,    0,    false, 0, 0,   true,  false },
-	{ _T("."),  false, '[',  ']',  false, 0, '^', false, false },
-	{ _T("\\/"), false, 0,    0,    false, 0, 0,   true,  false },
-	{ _T("."),  false, '\'', '\'', true,  1, 0,   false, false },
-	{ _T("/"),  true,  0,    0,    false, 0, 0,   true,  false },
-	{ _T("/"),  true,  0,    0,    false, 0, 0,   true,  false }, // Same as Unix
-	{ _T("."),  false, 0,    0,    false, 0, 0,   false, false },
-	{ _T("\\"), true,  0,    0,    false, 0, 0,   true,  false },
-	{ _T("/"),  true,  0,    0,    false, 0, 0,   true,  true  } // Cygwin is like Unix but has optional prefix of form "//server"
+	{ _T("/"),   true,     0,    0,    false, 0, 0,   true,  false }, // Failsafe
+	{ _T("/"),   true,     0,    0,    false, 0, 0,   true,  false },
+	{ _T("."),   false,  '[',  ']',    false, 0, '^', false, false },
+	{ _T("\\/"), false,    0,    0,    false, 0, 0,   true,  false },
+	{ _T("."),   false, '\'', '\'',     true, 1, 0,   false, false },
+	{ _T("/"),   true,     0,    0,    false, 0, 0,   true,  false },
+	{ _T("/"),   true,     0,    0,    false, 0, 0,   true,  false }, // Same as Unix
+	{ _T("."),   false,    0,    0,    false, 0, 0,   false, false },
+	{ _T("\\"),  true,     0,    0,    false, 0, 0,   true,  false },
+	{ _T("/"),   true,     0,    0,    false, 0, 0,   true,  true  }  // Cygwin is like Unix but has optional prefix of form "//server"
 };
 
 CServerPathData::CServerPathData()
@@ -34,9 +35,9 @@ CServerPathData::CServerPathData()
 }
 
 CServerPathData::CServerPathData(const CServerPathData& v)
+	: m_segments(v.m_segments)
+	, m_prefix(v.m_prefix)
 {
-	m_prefix = v.m_prefix;
-	m_segments = v.m_segments;
 }
 
 bool CServerPathData::operator==(const CServerPathData& cmp) const
@@ -51,17 +52,16 @@ bool CServerPathData::operator==(const CServerPathData& cmp) const
 }
 
 CServerPath::CServerPath()
+	: m_bEmpty(true)
+	, m_type(DEFAULT)
 {
-	m_type = DEFAULT;
-	m_bEmpty = true;
 }
 
 CServerPath::CServerPath(const CServerPath &path, wxString subdir)
-	: m_data(path.m_data)
+	: m_bEmpty(path.m_bEmpty)
+	, m_type(path.m_type)
+	, m_data(path.m_data)
 {
-	m_type = path.m_type;
-	m_bEmpty = path.m_bEmpty;
-
 	if (subdir == _T(""))
 		return;
 
@@ -70,16 +70,15 @@ CServerPath::CServerPath(const CServerPath &path, wxString subdir)
 }
 
 CServerPath::CServerPath(const CServerPath &path)
-	: m_data(path.m_data)
+	: m_bEmpty(path.m_bEmpty)
+	, m_type(path.m_type)
+	, m_data(path.m_data)
 {
-	m_type = path.m_type;
-	m_bEmpty = path.m_bEmpty;
 }
 
 CServerPath::CServerPath(wxString path, ServerType type /*=DEFAULT*/)
+	: m_type(type)
 {
-	m_type = type;
-
 	SetPath(path);
 }
 
@@ -236,26 +235,74 @@ wxString CServerPath::GetLastSegment() const
 		return _T("");
 }
 
+// libc sprintf can be so slow at times...
+wxChar* fast_sprint_number(wxChar* s, size_t n)
+{
+	wxChar tmp[20]; // Long enough for 2^64-1
+
+	wxChar* c = tmp;
+	do
+	{
+		*(c++) = n % 10 + '0';
+		n /= 10;
+	} while( n > 0 );
+
+	do
+	{
+		*(s++) = *(--c);
+	} while (c != tmp);
+
+	return s;
+}
+
+#define tstrcpy wcscpy
+
 wxString CServerPath::GetSafePath() const
 {
 	if (m_bEmpty)
 		return _T("");
 
+	#define INTLENGTH 20 // 2^64 - 1
+
+	int len = 5 // Type and 2x' ' and terminating 0
+		+ INTLENGTH; // Max length of prefix
+
+	len += m_data->m_prefix.size();
+	for (tConstSegmentIter iter = m_data->m_segments.begin(); iter != m_data->m_segments.end(); ++iter)
+		len += iter->Length() + 2 + INTLENGTH;
+
 	wxString safepath;
-	safepath.Printf(_T("%d %d "), m_type, (int)m_data->m_prefix.Length());
-	if (m_data->m_prefix != _T(""))
-		safepath += m_data->m_prefix + _T(" ");
+	wxChar* start = safepath.GetWriteBuf(len);
+	wxChar* t = start;
+
+	t = fast_sprint_number(t, m_type);
+	*(t++) = ' ';
+	t = fast_sprint_number(t, m_data->m_prefix.size());
+
+	if (!m_data->m_prefix.empty())
+	{
+		*(t++) = ' ';
+		tstrcpy(t, m_data->m_prefix);
+		t += m_data->m_prefix.size();
+	}
 
 	for (tConstSegmentIter iter = m_data->m_segments.begin(); iter != m_data->m_segments.end(); ++iter)
-		safepath += wxString::Format(_T("%d %s "), (int)iter->Length(), iter->c_str());
+	{
+		*(t++) = ' ';
+		t = fast_sprint_number(t, iter->size());
+		*(t++) = ' ';
+		tstrcpy(t, *iter);
+		t += iter->size();
+	}
+	*t = 0;
 
-	if (!m_data->m_segments.empty())
-		safepath.RemoveLast();
+	safepath.UngetWriteBuf( t - start );
+	safepath.Shrink();
 
 	return safepath;
 }
 
-bool CServerPath::SetSafePath(const wxString& path)
+bool CServerPath::SetSafePath(const wxString& path, bool coalesce)
 {
 	CServerPathData& data = m_data.Get();
 	m_bEmpty = true;
@@ -268,7 +315,7 @@ bool CServerPath::SetSafePath(const wxString& path)
 	// from file
 	const int len = (int)path.Len();
 	wxChar* begin = new wxChar[len + 1];
-	CSharedPointerArray<wxChar> tmp(begin);
+	CScopedArray<wxChar> tmp(begin);
 	memcpy(begin, (const wxChar*)path, (len + 1) * sizeof(wxChar));
 	wxChar* p = begin;
 
@@ -322,10 +369,12 @@ bool CServerPath::SetSafePath(const wxString& path)
 	{
 		*(p + prefix_len) = 0;
 		data.m_prefix = p;
+		if (coalesce)
+			::Coalesce(data.m_prefix);
 
 		p += prefix_len + 1;
 	}
-	
+
 	while (len > (p - begin))
 	{
 		int segment_len = 0;
@@ -349,7 +398,10 @@ bool CServerPath::SetSafePath(const wxString& path)
 		if (len - (p - begin) < segment_len)
 			return false;
 		*(p + segment_len) = 0;
-		data.m_segments.push_back(p);
+		wxString s(p);
+		if (coalesce)
+			::Coalesce(s);
+		data.m_segments.push_back(s);
 
 		p += segment_len + 1;
 	}
@@ -737,7 +789,7 @@ bool CServerPath::operator<(const CServerPath &op) const
 	else if (m_type < op.m_type)
 		return true;
 
-	std::list<wxString>::const_iterator iter1, iter2;
+	tConstSegmentIter iter1, iter2;
 	for (iter1 = m_data->m_segments.begin(), iter2 = op.m_data->m_segments.begin(); iter1 != m_data->m_segments.end(); ++iter1, ++iter2)
 	{
 		if (iter2 == op.m_data->m_segments.end())
@@ -868,9 +920,9 @@ CServerPath CServerPath::GetCommonParent(const CServerPath& path) const
 	parent.m_type = m_type;
 
 	CServerPathData& parentData = parent.m_data.Get();
-	
-	std::list<wxString>::const_iterator last = m_data->m_segments.end();
-	std::list<wxString>::const_iterator last2 = path.m_data->m_segments.end();
+
+	tConstSegmentIter last = m_data->m_segments.end();
+	tConstSegmentIter last2 = path.m_data->m_segments.end();
 	if (traits[m_type].prefixmode == 1)
 	{
 		if (m_data->m_prefix.empty())
@@ -882,8 +934,8 @@ CServerPath CServerPath::GetCommonParent(const CServerPath& path) const
 	else
 		parentData.m_prefix = m_data->m_prefix;
 
-	std::list<wxString>::const_iterator iter = m_data->m_segments.begin();
-	std::list<wxString>::const_iterator iter2 = path.m_data->m_segments.begin();
+	tConstSegmentIter iter = m_data->m_segments.begin();
+	tConstSegmentIter iter2 = path.m_data->m_segments.begin();
 	while (iter != last && iter2 != last2)
 	{
 		if (*iter != *iter2)
@@ -1000,4 +1052,12 @@ void CServerPath::EscapeSeparators(ServerType type, wxString& subdir)
 		for (const wxChar* p = traits[type].separators; *p; ++p)
 			subdir.Replace((wxString)*p, (wxString)traits[type].separatorEscape + traits[type].separators[0]);
 	}
-}
\ No newline at end of file
+}
+
+void CServerPath::Coalesce()
+{
+	CServerPathData& data = m_data.Get();
+	::Coalesce(data.m_prefix);
+	for (tSegmentIter iter = data.m_segments.begin(); iter != data.m_segments.end(); ++iter)
+		::Coalesce(*iter);
+}
