@@ -549,6 +549,9 @@ int CSftpControlSocket::ConnectSend()
 			case CProxySocket::SOCKS5:
 				type = 2;
 				break;
+			case CProxySocket::SOCKS4:
+				type = 3;
+				break;
 			default:
 				LogMessage(__TFILE__, __LINE__, this, Debug_Warning, _T("Unsupported proxy type"));
 				DoClose(FZ_REPLY_INTERNALERROR);
@@ -1918,22 +1921,21 @@ int CSftpControlSocket::FileTransferParseResponse(bool successful, const wxStrin
 
 		if (m_pEngine->GetOptions()->GetOptionVal(OPTION_PRESERVE_TIMESTAMPS))
 		{
-			wxFileName fn(pData->localFile);
-			if (fn.FileExists())
+			if (pData->download)
 			{
-				if (pData->download)
+				if (pData->fileTime.IsValid())
 				{
-					if (pData->fileTime.IsValid())
-						fn.SetTimes(&pData->fileTime, &pData->fileTime, 0);
+					if (!CLocalFileSystem::SetModificationTime(pData->localFile, pData->fileTime))
+						LogMessage(__TFILE__, __LINE__, this, Debug_Warning, _T("Could not set modification time"));
 				}
-				else
+			}
+			else
+			{
+				pData->fileTime = CLocalFileSystem::GetModificationTime(pData->localFile);
+				if (pData->fileTime.IsValid())
 				{
-					pData->fileTime = fn.GetModificationTime();
-					if (pData->fileTime.IsValid())
-					{
-						pData->opState = filetransfer_chmtime;
-						return SendNextCommand();
-					}
+					pData->opState = filetransfer_chmtime;
+					return SendNextCommand();
 				}
 			}
 		}
