@@ -8,7 +8,6 @@
 #include "filter.h"
 #include "inputdialog.h"
 #include <algorithm>
-#include <wx/dnd.h>
 #include "dndobjects.h"
 #include "Options.h"
 #ifdef __WXMSW__
@@ -27,11 +26,13 @@
 #define new DEBUG_NEW
 #endif
 
-class CLocalListViewDropTarget : public wxDropTarget
+
+class CLocalListViewDropTarget : public CListCtrlDropTarget
 {
 public:
 	CLocalListViewDropTarget(CLocalListView* pLocalListView)
-		: m_pLocalListView(pLocalListView), m_pFileDataObject(new wxFileDataObject()),
+		: CListCtrlDropTarget(pLocalListView)
+		, m_pLocalListView(pLocalListView), m_pFileDataObject(new wxFileDataObject()),
 		m_pRemoteDataObject(new CRemoteDataObject())
 	{
 		m_pDataObject = new wxDataObjectComposite;
@@ -104,7 +105,7 @@ public:
 				return wxDragNone;
 			}
 
-			if (!m_pLocalListView->m_pState->GetServer() || m_pRemoteDataObject->GetServer() != *m_pLocalListView->m_pState->GetServer())
+			if (!m_pLocalListView->m_pState->GetServer() || !m_pRemoteDataObject->GetServer().EqualsNoPass(*m_pLocalListView->m_pState->GetServer()))
 			{
 				wxMessageBox(_("Drag&drop between different servers has not been implemented yet."));
 				return wxDragNone;
@@ -119,6 +120,7 @@ public:
 
 	virtual bool OnDrop(wxCoord x, wxCoord y)
 	{
+		CListCtrlDropTarget::OnDrop(x, y);
 		ClearDropHighlight();
 
 		if (m_pLocalListView->m_fileData.empty())
@@ -127,7 +129,12 @@ public:
 		return true;
 	}
 
-	wxString DisplayDropHighlight(wxPoint point)
+	virtual void DisplayDropHighlight(wxPoint point)
+	{
+		DoDisplayDropHighlight(point);
+	}
+
+	virtual wxString DoDisplayDropHighlight(wxPoint point)
 	{
 		wxString subDir;
 
@@ -174,6 +181,8 @@ public:
 
 	virtual wxDragResult OnDragOver(wxCoord x, wxCoord y, wxDragResult def)
 	{
+		CListCtrlDropTarget::OnDragOver(x, y, def);
+
 		if (def == wxDragError ||
 			def == wxDragNone ||
 			def == wxDragCancel)
@@ -188,7 +197,7 @@ public:
 			return wxDragNone;
 		}
 
-		const wxString& subdir = DisplayDropHighlight(wxPoint(x, y));
+		const wxString& subdir = DoDisplayDropHighlight(wxPoint(x, y));
 
 		CLocalPath dir = m_pLocalListView->m_pState->GetLocalDir();
 		if (subdir == _T(""))
@@ -214,11 +223,13 @@ public:
 
 	virtual void OnLeave()
 	{
+		CListCtrlDropTarget::OnLeave();
 		ClearDropHighlight();
 	}
 
 	virtual wxDragResult OnEnter(wxCoord x, wxCoord y, wxDragResult def)
 	{
+		CListCtrlDropTarget::OnEnter(x, y, def);
 		return OnDragOver(x, y, def);
 	}
 
@@ -254,6 +265,7 @@ CLocalListView::CLocalListView(wxWindow* pParent, CState *pState, CQueueView *pQ
 	: CFileListCtrl<CLocalFileData>(pParent, pState, pQueue),
 	CStateEventHandler(pState)
 {
+	wxGetApp().AddStartupProfileRecord(_T("CLocalListView::CLocalListView"));
 	m_pState->RegisterHandler(this, STATECHANGE_LOCAL_DIR);
 	m_pState->RegisterHandler(this, STATECHANGE_APPLYFILTER);
 	m_pState->RegisterHandler(this, STATECHANGE_LOCAL_REFRESH_FILE);
@@ -299,6 +311,7 @@ bool CLocalListView::DisplayDir(wxString dirname)
 
 	wxString focused;
 	std::list<wxString> selectedNames;
+	bool ensureVisible = false;
 	if (m_dir != dirname)
 	{
 		ResetSearchPrefix();
@@ -307,7 +320,10 @@ bool CLocalListView::DisplayDir(wxString dirname)
 			ExitComparisonMode();
 
 		ClearSelection();
-		focused = _T("..");
+		focused = m_pState->GetPreviouslyVisitedLocalSubdir();
+		ensureVisible = !focused.IsEmpty();
+		if (focused.IsEmpty())
+			focused = _T("..");
 
 		if (GetItemCount())
 			EnsureVisible(0);
@@ -443,7 +459,7 @@ regular_dir:
 		RefreshComparison();
 	}
 
-	ReselectItems(selectedNames, focused);
+	ReselectItems(selectedNames, focused, ensureVisible);
 
 	RefreshListOnly();
 
@@ -483,7 +499,7 @@ void CLocalListView::OnItemActivated(wxListEvent &event)
 	bool back = false;
 
 	int item = -1;
-	while (true)
+	for (;;)
 	{
 		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 		if (item == -1)
@@ -581,7 +597,7 @@ void CLocalListView::OnItemActivated(wxListEvent &event)
 
 	const bool queue_only = action == 1;
 
-	m_pQueue->QueueFile(queue_only, false, CLocalPath(m_dir), data->name, data->name, path, *pServer, data->size);
+	m_pQueue->QueueFile(queue_only, false, data->name, wxEmptyString, CLocalPath(m_dir), path, *pServer, data->size);
 	m_pQueue->QueueFile_Finish(true);
 }
 
@@ -791,7 +807,7 @@ public:
 	{
 	}
 
-	virtual ~CLocalListViewSort() { }
+	virtual ~CLocalListViewSort() {}
 
 	virtual bool operator()(int a, int b) const = 0;
 
@@ -1074,16 +1090,13 @@ void CLocalListView::OnContextMenu(wxContextMenuEvent& event)
 		const CLocalFileData* const data = GetData(index);
 		if (!data || (!index && m_hasParent))
 		{
-			pMenu->Enable(XRCID("ID_UPLOAD"), false);
-			pMenu->Enable(XRCID("ID_ADDTOQUEUE"), false);
-			pMenu->Enable(XRCID("ID_DELETE"), false);
-			pMenu->Enable(XRCID("ID_RENAME"), false);
 			pMenu->Enable(XRCID("ID_OPEN"), false);
+			pMenu->Enable(XRCID("ID_RENAME"), false);
 			pMenu->Enable(XRCID("ID_EDIT"), false);
 		}
-		if (data->flags == fill)
+		if (data && data->flags == fill || (!index && m_hasParent))
 			fillCount++;
-		if (data->dir)
+		if (data && data->dir)
 			selectedDir = true;
 		index = GetNextItem(index, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 	}
@@ -1116,17 +1129,6 @@ void CLocalListView::OnContextMenu(wxContextMenuEvent& event)
 
 void CLocalListView::OnMenuUpload(wxCommandEvent& event)
 {
-	long item = -1;
-	while (true)
-	{
-		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
-		if (item == -1)
-			break;
-
-		if (!item && m_hasParent)
-			return;
-	}
-
 	const CServer* pServer = m_pState->GetServer();
 	if (!pServer)
 	{
@@ -1138,10 +1140,12 @@ void CLocalListView::OnMenuUpload(wxCommandEvent& event)
 
 	bool queue_only = event.GetId() == XRCID("ID_ADDTOQUEUE");
 
-	item = -1;
-	while (true)
+	long item = -1;
+	for (;;)
 	{
 		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
+		if (!item && m_hasParent)
+			continue;
 		if (item == -1)
 			break;
 
@@ -1169,7 +1173,7 @@ void CLocalListView::OnMenuUpload(wxCommandEvent& event)
 		}
 		else
 		{
-			m_pQueue->QueueFile(queue_only, false, CLocalPath(m_dir), data->name, data->name, path, *pServer, data->size);
+			m_pQueue->QueueFile(queue_only, false, data->name, wxEmptyString, CLocalPath(m_dir), path, *pServer, data->size);
 			added = true;
 		}
 	}
@@ -1482,7 +1486,7 @@ std::list<wxString> CLocalListView::RememberSelectedItems(wxString& focused)
 #endif
 	{
 		int item = -1;
-		while (true)
+		for (;;)
 		{
 			item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 			if (item == -1)
@@ -1512,7 +1516,7 @@ std::list<wxString> CLocalListView::RememberSelectedItems(wxString& focused)
 	return selectedNames;
 }
 
-void CLocalListView::ReselectItems(const std::list<wxString>& selectedNames, wxString focused)
+void CLocalListView::ReselectItems(const std::list<wxString>& selectedNames, wxString focused, bool ensureVisible)
 {
 	// Reselect previous items if neccessary.
 	// Sorting direction did not change. We just have to scan through items once
@@ -1527,6 +1531,8 @@ void CLocalListView::ReselectItems(const std::list<wxString>& selectedNames, wxS
 			if (data.name == focused)
 			{
 				SetItemState(i, wxLIST_STATE_FOCUSED, wxLIST_STATE_FOCUSED);
+				if (ensureVisible)
+					EnsureVisible(i);
 				return;
 			}
 		}
@@ -1536,7 +1542,7 @@ void CLocalListView::ReselectItems(const std::list<wxString>& selectedNames, wxS
 	int firstSelected = -1;
 
 	int i = -1;
-	for (std::list<wxString>::const_iterator iter = selectedNames.begin(); iter != selectedNames.end(); iter++)
+	for (std::list<wxString>::const_iterator iter = selectedNames.begin(); iter != selectedNames.end(); ++iter)
 	{
 		while (++i < (int)m_indexMapping.size())
 		{
@@ -1544,6 +1550,8 @@ void CLocalListView::ReselectItems(const std::list<wxString>& selectedNames, wxS
 			if (data.name == focused)
 			{
 				SetItemState(i, wxLIST_STATE_FOCUSED, wxLIST_STATE_FOCUSED);
+				if (ensureVisible)
+					EnsureVisible(i);
 				focused = _T("");
 			}
 			if (data.dir && *iter == (_T("d") + data.name))
@@ -1593,7 +1601,7 @@ void CLocalListView::OnStateChange(CState* pState, enum t_statechange_notificati
 void CLocalListView::OnBeginDrag(wxListEvent& event)
 {
 	long item = -1;
-	while (true)
+	for (;;)
 	{
 		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 		if (item == -1)
@@ -1610,7 +1618,7 @@ void CLocalListView::OnBeginDrag(wxListEvent& event)
 	pDragDropManager->localParent = m_dir;
 
 	item = -1;
-	while (true)
+	for (;;)
 	{
 		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 		if (item == -1)
@@ -1678,7 +1686,7 @@ void CLocalListView::RefreshFile(const wxString& file)
 
 	// Look if file data already exists
 	unsigned int i = 0;
-	for (std::vector<CLocalFileData>::iterator iter = m_fileData.begin(); iter != m_fileData.end(); iter++, i++)
+	for (std::vector<CLocalFileData>::iterator iter = m_fileData.begin(); iter != m_fileData.end(); ++iter, ++i)
 	{
 		const CLocalFileData& oldData = *iter;
 		if (oldData.name != file)
@@ -1693,7 +1701,7 @@ void CLocalListView::RefreshFile(const wxString& file)
 #endif
 			{
 				int item = -1;
-				while (true)
+				for (;;)
 				{
 					item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 					if (item == -1)
@@ -1763,7 +1771,7 @@ void CLocalListView::RefreshFile(const wxString& file)
 	// Find correct position in index mapping
 	std::vector<unsigned int>::iterator start = m_indexMapping.begin();
 	if (m_hasParent)
-		start++;
+		++start;
 	CFileListCtrl<CLocalFileData>::CSortComparisonObject compare = GetSortComparisonObject();
 	std::vector<unsigned int>::iterator insertPos = std::lower_bound(start, m_indexMapping.end(), index, compare);
 	compare.Destroy();
@@ -2011,7 +2019,7 @@ void CLocalListView::OnMenuEdit(wxCommandEvent& event)
 			return;
 	}
 
-	for (std::list<CLocalFileData>::const_iterator data = selected_item_list.begin(); data != selected_item_list.end(); data++)
+	for (std::list<CLocalFileData>::const_iterator data = selected_item_list.begin(); data != selected_item_list.end(); ++data)
 	{
 		wxFileName fn(m_dir, data->name);
 
@@ -2082,7 +2090,7 @@ void CLocalListView::OnMenuOpen(wxCommandEvent& event)
 	long item = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 	if (item == -1)
 	{
-		wxLaunchDefaultBrowser(CState::GetAsURL(m_dir));
+		CState::OpenInFileManager(m_dir);
 		return;
 	}
 
@@ -2131,7 +2139,7 @@ void CLocalListView::OnMenuOpen(wxCommandEvent& event)
 			return;
 	}
 
-	for (std::list<CLocalFileData>::const_iterator data = selected_item_list.begin(); data != selected_item_list.end(); data++)
+	for (std::list<CLocalFileData>::const_iterator data = selected_item_list.begin(); data != selected_item_list.end(); ++data)
 	{
 		if (data->dir)
 		{
@@ -2142,7 +2150,7 @@ void CLocalListView::OnMenuOpen(wxCommandEvent& event)
 				continue;
 			}
 
-			wxLaunchDefaultBrowser(CState::GetAsURL(path.GetPath()));
+			CState::OpenInFileManager(path.GetPath());
 			continue;
 		}
 
@@ -2204,7 +2212,7 @@ void CLocalListView::OnVolumesEnumerated(wxCommandEvent& event)
 	if (m_dir != _T("\\"))
 		return;
 
-	for (std::list<CVolumeDescriptionEnumeratorThread::t_VolumeInfo>::const_iterator iter = volumeInfo.begin(); iter != volumeInfo.end(); iter++)
+	for (std::list<CVolumeDescriptionEnumeratorThread::t_VolumeInfo>::const_iterator iter = volumeInfo.begin(); iter != volumeInfo.end(); ++iter)
 	{
 		wxString drive = iter->volume;
 
@@ -2219,7 +2227,7 @@ void CLocalListView::OnVolumesEnumerated(wxCommandEvent& event)
 			continue;
 
 		m_fileData[index].label = drive + _T(" (") + iter->volumeName + _T(")");
-		
+
 		RefreshItem(item);
 	}
 }
