@@ -73,6 +73,7 @@ BEGIN_EVENT_TABLE(CMainFrame, wxFrame)
 	EVT_SIZE(CMainFrame::OnSize)
 	EVT_MENU(wxID_ANY, CMainFrame::OnMenuHandler)
 	EVT_FZ_NOTIFICATION(wxID_ANY, CMainFrame::OnEngineEvent)
+	EVT_COMMAND(wxID_ANY, fzEVT_UPDATE_LED_TOOLTIP, CMainFrame::OnUpdateLedTooltip)
 	EVT_TOOL(XRCID("ID_TOOLBAR_DISCONNECT"), CMainFrame::OnDisconnect)
 	EVT_MENU(XRCID("ID_MENU_SERVER_DISCONNECT"), CMainFrame::OnDisconnect)
 	EVT_TOOL(XRCID("ID_TOOLBAR_CANCEL"), CMainFrame::OnCancel)
@@ -206,7 +207,9 @@ protected:
 };
 
 CMainFrame::CMainFrame()
+	: m_comparisonToggleAcceleratorId(wxNewId())
 {
+	wxGetApp().AddStartupProfileRecord(_T("CMainFrame::CMainFrame"));
 	wxRect screen_size = CWindowStateManager::GetScreenDimensions();
 
 	wxSize initial_size;
@@ -334,8 +337,6 @@ CMainFrame::CMainFrame()
 
 	m_pContextControl->CreateTab();
 
-	m_pContextControl->GetCurrentControls();
-	
 	switch (message_log_position)
 	{
 	case 1:
@@ -412,12 +413,13 @@ CMainFrame::CMainFrame()
 		SetDefaultSplitterPositions();
 
 	wxAcceleratorEntry entries[11];
-	entries[0].Set(wxACCEL_CMD | wxACCEL_SHIFT, 'I', XRCID("ID_MENU_VIEW_FILTERS"));
+	//entries[0].Set(wxACCEL_CMD | wxACCEL_SHIFT, 'I', XRCID("ID_MENU_VIEW_FILTERS"));
 	for (int i = 0; i < 10; i++)
 	{
 		tab_hotkey_ids[i] = wxNewId();
-		entries[i + 1].Set(wxACCEL_CMD, (int)'0' + i, tab_hotkey_ids[i]);
+		entries[i].Set(wxACCEL_CMD, (int)'0' + i, tab_hotkey_ids[i]);
 	}
+	entries[10].Set(wxACCEL_CMD | wxACCEL_SHIFT, 'O', m_comparisonToggleAcceleratorId);
 
 	wxAcceleratorTable accel(sizeof(entries) / sizeof(wxAcceleratorEntry), entries);
 	SetAcceleratorTable(accel);
@@ -500,13 +502,14 @@ void CMainFrame::OnSize(wxSizeEvent &event)
 
 bool CMainFrame::CreateMenus()
 {
+	wxGetApp().AddStartupProfileRecord(_T("CMainFrame::CreateMenus"));
 	if (m_pMenuBar)
 	{
 		SetMenuBar(0);
 		delete m_pMenuBar;
 	}
 	m_pMenuBar = CMenuBar::Load(this);
-	
+
 	if (!m_pMenuBar)
 		return false;
 
@@ -521,6 +524,7 @@ bool CMainFrame::CreateMenus()
 
 bool CMainFrame::CreateQuickconnectBar()
 {
+	wxGetApp().AddStartupProfileRecord(_T("CMainFrame::CreateQuickconnectBar"));
 	delete m_pQuickconnectBar;
 
 	m_pQuickconnectBar = new CQuickconnectBar();
@@ -621,6 +625,14 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 		int *x = 0;
 		*x = 0;
 	}
+	else if (event.GetId() == XRCID("ID_CIPHERS"))
+	{
+		CInputDialog dlg;
+		dlg.Create(this, _T("Ciphers"), _T("Priority string:"));
+		dlg.AllowEmpty(true);
+		if (dlg.ShowModal() == wxID_OK)
+			wxMessageBox(ListTlsCiphers(dlg.GetValue()), _T("Ciphers"));
+	}
 	else if (event.GetId() == XRCID("ID_CLEARCACHE_LAYOUT"))
 	{
 		CWrapEngine::ClearCache();
@@ -672,7 +684,7 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 
 		COptions::Get()->SetOption(OPTION_VIEW_HIDDEN_FILES, showHidden ? 1 : 0);
 		const std::vector<CState*> *pStates = CContextManager::Get()->GetAllStates();
-		for (std::vector<CState*>::const_iterator iter = pStates->begin(); iter != pStates->end(); iter++)
+		for (std::vector<CState*>::const_iterator iter = pStates->begin(); iter != pStates->end(); ++iter)
 		{
 			CState* pState = *iter;
 			CServerPath path = pState->GetRemotePath();
@@ -726,7 +738,7 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 	else if (event.GetId() == XRCID("ID_MENU_HELP_GETTINGHELP") ||
 			 event.GetId() == XRCID("ID_MENU_HELP_BUGREPORT"))
 	{
-		wxString url(_T("http://filezilla-project.org/support.php?type=client&mode="));
+		wxString url(_T("https://filezilla-project.org/support.php?type=client&mode="));
 		if (event.GetId() == XRCID("ID_MENU_HELP_GETTINGHELP"))
 			url += _T("help");
 		else
@@ -852,7 +864,7 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 			CSiteManager::GetBookmarks(controls->site_bookmarks->path, controls->site_bookmarks->bookmarks);
 			if (m_pMenuBar)
 				m_pMenuBar->UpdateBookmarkMenu();
-		}	
+		}
 	}
 	else if (event.GetId() == XRCID("ID_MENU_HELP_WELCOME"))
 	{
@@ -878,6 +890,19 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 		CSpeedLimitsDialog dlg;
 		dlg.Run(this);
 	}
+	else if (event.GetId() == m_comparisonToggleAcceleratorId)
+	{
+		CState* pState = CContextManager::Get()->GetCurrentContext();
+		if (!pState)
+			return;
+
+		int old_mode = COptions::Get()->GetOptionVal(OPTION_COMPARISONMODE);
+		COptions::Get()->SetOption(OPTION_COMPARISONMODE, old_mode ? 0 : 1);
+
+		CComparisonManager* pComparisonManager = pState->GetComparisonManager();
+		if (pComparisonManager && pComparisonManager->IsComparing())
+			pComparisonManager->CompareListings();
+	}
 	else
 	{
 		for (int i = 0; i < 10; i++)
@@ -896,8 +921,7 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 			return;
 		}
 
-		wxString path;
-		CSiteManagerItemData_Site* pData = CSiteManager::GetSiteById(event.GetId(), path);
+		CSiteManagerItemData_Site* pData = CSiteManager::GetSiteById(event.GetId());
 
 		if (!pData)
 		{
@@ -905,15 +929,7 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 			return;
 		}
 
-		if (!ConnectToSite(pData))
-		{
-			delete pData;
-			return;
-		}
-
-		SetBookmarksFromPath(path);
-		if (m_pMenuBar)
-			m_pMenuBar->UpdateBookmarkMenu();
+		ConnectToSite(pData);
 
 		delete pData;
 	}
@@ -927,11 +943,11 @@ void CMainFrame::OnEngineEvent(wxEvent &event)
 
 	const std::vector<CState*> *pStates = CContextManager::Get()->GetAllStates();
 	CState* pState = 0;
-	for (std::vector<CState*>::const_iterator iter = pStates->begin(); iter != pStates->end(); iter++)
+	for (std::vector<CState*>::const_iterator iter = pStates->begin(); iter != pStates->end(); ++iter)
 	{
 		if ((*iter)->m_pEngine != pEngine)
 			continue;
-		
+
 		pState = *iter;
 		break;
 	}
@@ -985,7 +1001,7 @@ void CMainFrame::OnEngineEvent(wxEvent &event)
 			{
 				CAsyncRequestNotification* pAsyncRequest = reinterpret_cast<CAsyncRequestNotification *>(pNotification);
 				if (pAsyncRequest->GetRequestID() == reqId_fileexists)
-					m_pQueueView->ProcessNotification(pNotification);
+					m_pQueueView->ProcessNotification(pEngine, pNotification);
 				else
 				{
 					if (pAsyncRequest->GetRequestID() == reqId_certificate)
@@ -1002,7 +1018,7 @@ void CMainFrame::OnEngineEvent(wxEvent &event)
 			}
 			break;
 		case nId_transferstatus:
-			m_pQueueView->ProcessNotification(pNotification);
+			m_pQueueView->ProcessNotification(pEngine, pNotification);
 			break;
 		case nId_sftp_encryption:
 			{
@@ -1027,8 +1043,32 @@ void CMainFrame::OnEngineEvent(wxEvent &event)
 	}
 }
 
+void CMainFrame::OnUpdateLedTooltip(wxCommandEvent& event)
+{
+	wxString tooltipText;
+
+	wxFileOffset downloadSpeed = m_pQueueView->GetCurrentDownloadSpeed();
+	wxFileOffset uploadSpeed = m_pQueueView->GetCurrentUploadSpeed();
+
+	CSizeFormat::_format format = static_cast<CSizeFormat::_format>(COptions::Get()->GetOptionVal(OPTION_SIZE_FORMAT));
+	if (format == CSizeFormat::bytes)
+		format = CSizeFormat::iec;
+
+	const wxString downloadSpeedStr = CSizeFormat::Format(downloadSpeed, true, format,
+														  COptions::Get()->GetOptionVal(OPTION_SIZE_USETHOUSANDSEP) != 0,
+														  COptions::Get()->GetOptionVal(OPTION_SIZE_DECIMALPLACES));
+	const wxString uploadSpeedStr = CSizeFormat::Format(uploadSpeed, true, format,
+														COptions::Get()->GetOptionVal(OPTION_SIZE_USETHOUSANDSEP) != 0,
+														COptions::Get()->GetOptionVal(OPTION_SIZE_DECIMALPLACES));
+	tooltipText.Printf(_("Download speed: %s/s\nUpload speed: %s/s"), downloadSpeedStr.c_str(), uploadSpeedStr.c_str());
+
+	m_pActivityLed[0]->SetToolTip(tooltipText);
+	m_pActivityLed[1]->SetToolTip(tooltipText);
+}
+
 bool CMainFrame::CreateToolBar()
 {
+	wxGetApp().AddStartupProfileRecord(_T("CMainFrame::CreateToolBar"));
 	if (m_pToolBar)
 	{
 #ifdef __WXMAC__
@@ -1103,6 +1143,79 @@ BOOL CALLBACK FzEnumThreadWndProc(HWND hwnd, LPARAM lParam)
 }
 #endif //__WXMSW__
 
+
+bool CMainFrame::CloseDialogsAndQuit(wxCloseEvent &event)
+{
+#ifndef __WXMAC__
+	if (m_taskBarIcon)
+	{
+		delete m_taskBarIcon;
+		m_taskBarIcon = 0;
+		m_closeEvent = event.GetEventType();
+		m_closeEventTimer.Start(1, true);
+		return false;
+	}
+#endif
+
+	// We need to close all other top level windows on the stack before closing the main frame.
+	// In other words, all open dialogs need to be closed.
+	static int prev_size = 0;
+
+	int size = wxTopLevelWindows.size();
+	static wxTopLevelWindow* pLast = 0;
+	if (wxTopLevelWindows.size())
+	{
+		wxWindowList::reverse_iterator iter = wxTopLevelWindows.rbegin();
+		wxTopLevelWindow* pTop = (wxTopLevelWindow*)(*iter);
+		while (pTop != this && (size != prev_size || pLast != pTop))
+		{
+			wxDialog* pDialog = wxDynamicCast(pTop, wxDialog);
+			if (pDialog)
+				pDialog->EndModal(wxID_CANCEL);
+			else
+			{
+				wxWindow* pParent = pTop->GetParent();
+				if (m_pQueuePane && pParent == m_pQueuePane)
+				{
+					// It's the AUI frame manager hint window. Ignore it
+					++iter;
+					if (iter == wxTopLevelWindows.rend())
+						break;
+					pTop = (wxTopLevelWindow*)(*iter);
+					continue;
+				}
+				wxString title = pTop->GetTitle();
+				pTop->Destroy();
+			}
+
+			prev_size = size;
+			pLast = pTop;
+
+			m_closeEvent = event.GetEventType();
+			m_closeEventTimer.Start(1, true);
+
+			return false;
+		}
+	}
+
+#ifdef __WXMSW__
+	// wxMessageBox does not use wxTopLevelWindow, close it too
+	bool dialog = false;
+	EnumThreadWindows(GetCurrentThreadId(), FzEnumThreadWndProc, (LPARAM)&dialog);
+	if (dialog)
+	{
+		m_closeEvent = event.GetEventType();
+		m_closeEventTimer.Start(1, true);
+
+		return false;
+	}
+#endif //__WXMSW__
+
+	// At this point all other top level windows should be closed.
+	return true;
+}
+
+
 void CMainFrame::OnClose(wxCloseEvent &event)
 {
 	if (!m_bQuit)
@@ -1162,60 +1275,6 @@ void CMainFrame::OnClose(wxCloseEvent &event)
 			quit_confirmation_displayed = false;
 		}
 
-		if (!event.CanVeto())
-		{
-			// We need to close all other top level windows on the stack before closing the main frame.
-			// In other words, all open dialogs need to be closed.
-			static int prev_size = 0;
-
-			int size = wxTopLevelWindows.size();
-			static wxTopLevelWindow* pLast = 0;
-			wxWindowList::reverse_iterator iter = wxTopLevelWindows.rbegin();
-			wxTopLevelWindow* pTop = (wxTopLevelWindow*)(*iter);
-			while (pTop != this && (size != prev_size || pLast != pTop))
-			{
-				wxDialog* pDialog = wxDynamicCast(pTop, wxDialog);
-				if (pDialog)
-					pDialog->EndModal(wxID_CANCEL);
-				else
-				{
-					wxWindow* pParent = pTop->GetParent();
-					if (m_pQueuePane && pParent == m_pQueuePane)
-					{
-						// It's the AUI frame manager hint window. Ignore it
-						iter++;
-						pTop = (wxTopLevelWindow*)(*iter);
-						continue;
-					}
-					wxString title = pTop->GetTitle();
-					pTop->Destroy();
-				}
-
-				prev_size = size;
-				pLast = pTop;
-
-				m_closeEvent = event.GetEventType();
-				m_closeEventTimer.Start(1, true);
-
-				return;
-			}
-
-#ifdef __WXMSW__
-			// wxMessageBox does not use wxTopLevelWindow, close it too
-			bool dialog = false;
-			EnumThreadWindows(GetCurrentThreadId(), FzEnumThreadWndProc, (LPARAM)&dialog);
-			if (dialog)
-			{
-				m_closeEvent = event.GetEventType();
-				m_closeEventTimer.Start(1, true);
-
-				return;
-			}
-#endif //__WXMSW__
-
-			// At this point all other top level windows should be closed.
-		}
-
 		if (m_pWindowStateManager)
 		{
 			m_pWindowStateManager->Remember(OPTION_MAINWINDOW_POSITION);
@@ -1224,7 +1283,7 @@ void CMainFrame::OnClose(wxCloseEvent &event)
 		}
 
 		RememberSplitterPositions();
-		
+
 #ifdef __WXMAC__
 		if (m_pToolBar)
 			COptions::Get()->SetOption(OPTION_TOOLBAR_HIDDEN, m_pToolBar->IsShown() ? 0 : 1);
@@ -1233,6 +1292,8 @@ void CMainFrame::OnClose(wxCloseEvent &event)
 	}
 
 	Show(false);
+	if (!CloseDialogsAndQuit(event))
+		return;
 
 	// Getting deleted by wxWidgets
 	for (int i = 0; i < 2; i++)
@@ -1262,7 +1323,7 @@ void CMainFrame::OnClose(wxCloseEvent &event)
 
 	bool res = true;
 	const std::vector<CState*> *pStates = CContextManager::Get()->GetAllStates();
-	for (std::vector<CState*>::const_iterator iter = pStates->begin(); iter != pStates->end(); iter++)
+	for (std::vector<CState*>::const_iterator iter = pStates->begin(); iter != pStates->end(); ++iter)
 	{
 		CState* pState = *iter;
 		if (!pState->m_pCommandQueue)
@@ -1286,7 +1347,7 @@ void CMainFrame::OnClose(wxCloseEvent &event)
 		COptions::Get()->SetOption(OPTION_LAST_CONNECTED_SITE, controls->site_bookmarks->path);
 	}
 
-	for (std::vector<CState*>::const_iterator iter = pStates->begin(); iter != pStates->end(); iter++)
+	for (std::vector<CState*>::const_iterator iter = pStates->begin(); iter != pStates->end(); ++iter)
 	{
 		CState *pState = *iter;
 		pState->DestroyEngine();
@@ -1324,7 +1385,7 @@ void CMainFrame::OnReconnect(wxCommandEvent &event)
 	}
 
 	CServerPath path = pState->GetLastServerPath();
-	ConnectToServer(server, path);
+	ConnectToServer(server, path, true);
 }
 
 void CMainFrame::OnRefresh(wxCommandEvent &event)
@@ -1423,12 +1484,7 @@ void CMainFrame::OpenSiteManager(const CServer* pServer /*=0*/)
 		if (!dlg.GetServer(data))
 			return;
 
-		if (ConnectToSite(&data))
-		{
-			// Get new bookmarks
-			wxString path = dlg.GetSitePath();
-			SetBookmarksFromPath(path);
-		}
+		ConnectToSite(&data);
 	}
 
 	if (m_pMenuBar)
@@ -1566,7 +1622,7 @@ void CMainFrame::OnToggleLogView(wxCommandEvent& event)
 			shown = true;
 		}
 	}
-	
+
 	if (COptions::Get()->GetOptionVal(OPTION_MESSAGELOG_POSITION) != 2)
 		COptions::Get()->SetOption(OPTION_SHOW_MESSAGELOG, shown);
 }
@@ -1617,7 +1673,7 @@ void CMainFrame::ShowLocalTree()
 			continue;
 
 		controls->pLocalTreeViewPanel->SetHeader(controls->pLocalListViewPanel->DetachHeader());
-		
+
 		if (layout == 3 && swap)
 			controls->pLocalSplitter->SplitVertically(controls->pLocalListViewPanel, controls->pLocalTreeViewPanel);
 		else if (layout)
@@ -1781,12 +1837,12 @@ void CMainFrame::UpdateLayout(int layout /*=-1*/, int swap /*=-1*/, int messagel
 
 	if (messagelog_position == -1)
 		messagelog_position = COptions::Get()->GetOptionVal(OPTION_MESSAGELOG_POSITION);
-	
+
 	// First handle changes in message log position as it can make size of the other panes change
 	{
 		bool shown = m_pStatusView->IsShown();
 		wxWindow* parent = m_pStatusView->GetParent();
-	
+
 		bool changed;
 		if (parent == m_pTopSplitter && messagelog_position != 0)
 		{
@@ -1972,7 +2028,7 @@ void CMainFrame::OnSitemanagerDropdown(wxCommandEvent& event)
 	ShowDropdownMenu(pMenu, m_pToolBar, event);
 }
 
-bool CMainFrame::ConnectToSite(CSiteManagerItemData_Site* const pData)
+bool CMainFrame::ConnectToSite(CSiteManagerItemData_Site* const pData, bool newTab)
 {
 	wxASSERT(pData);
 
@@ -1983,6 +2039,9 @@ bool CMainFrame::ConnectToSite(CSiteManagerItemData_Site* const pData)
 			return false;
 	}
 
+	if (newTab)
+		m_pContextControl->CreateTab();
+
 	if (!ConnectToServer(pData->m_server, pData->m_remoteDir))
 		return false;
 
@@ -1999,6 +2058,10 @@ bool CMainFrame::ConnectToSite(CSiteManagerItemData_Site* const pData)
 		}
 	}
 
+	SetBookmarksFromPath(pData->m_path);
+	if (m_pMenuBar)
+		m_pMenuBar->UpdateBookmarkMenu();
+		
 	return true;
 }
 
@@ -2079,7 +2142,7 @@ void CMainFrame::OnNavigationKeyEvent(wxNavigationKeyEvent& event)
 	windowOrder.push_back(m_pQueuePane);
 
 	std::list<wxWindow*>::iterator iter;
-	for (iter = windowOrder.begin(); iter != windowOrder.end(); iter++)
+	for (iter = windowOrder.begin(); iter != windowOrder.end(); ++iter)
 	{
 		if (*iter == event.GetEventObject())
 			break;
@@ -2135,7 +2198,7 @@ void CMainFrame::OnChar(wxKeyEvent& event)
 	else
 	{
 		wxWindow *parent = focused->GetParent();
-		for (iter = windowOrder.begin(); iter != windowOrder.end(); iter++)
+		for (iter = windowOrder.begin(); iter != windowOrder.end(); ++iter)
 		{
 			if (*iter == focused || *iter == parent)
 			{
@@ -2162,7 +2225,7 @@ void CMainFrame::FocusNextEnabled(std::list<wxWindow*>& windowOrder, std::list<w
 		skipFirst = false;
 		if (forward)
 		{
-			iter++;
+			++iter;
 			if (iter == windowOrder.end())
 				iter = windowOrder.begin();
 		}
@@ -2170,7 +2233,7 @@ void CMainFrame::FocusNextEnabled(std::list<wxWindow*>& windowOrder, std::list<w
 		{
 			if (iter == windowOrder.begin())
 				iter = windowOrder.end();
-			iter--;
+			--iter;
 		}
 
 		if (iter == start)
@@ -2188,7 +2251,7 @@ void CMainFrame::RememberSplitterPositions()
 	CContextControl::_context_controls* controls = m_pContextControl->GetCurrentControls();
 	if (!controls)
 		return;
-	
+
 	wxString posString;
 
 	// top_pos
@@ -2383,7 +2446,7 @@ void CMainFrame::ShowDropdownMenu(wxMenu* pMenu, wxToolBar* pToolBar, wxCommandE
 	{
 #ifdef __WXMSW__
 		RECT r;
-        if (::SendMessage((HWND)pToolBar->GetHandle(), TB_GETITEMRECT, pToolBar->GetToolPos(event.GetId()), (LPARAM)&r))
+		if (::SendMessage((HWND)pToolBar->GetHandle(), TB_GETITEMRECT, pToolBar->GetToolPos(event.GetId()), (LPARAM)&r))
 			pToolBar->PopupMenu(pMenu, r.left, r.bottom);
 		else
 #endif
@@ -2430,11 +2493,33 @@ void CMainFrame::ProcessCommandLine()
 	if (!pCommandLine)
 		return;
 
+	wxString local;
+	if ((local = pCommandLine->GetOption(CCommandLine::local)) != _T(""))
+	{
+		
+		if (!wxDir::Exists(local))
+		{
+			wxString str = _("Path not found:");
+			str += _T("\n") + local;
+			wxMessageBox(str, _("Syntax error in command line"));
+			return;
+		}
+		
+		CState *pState = CContextManager::Get()->GetCurrentContext();
+		if (!pState)
+			return;
+			
+		pState->SetLocalDir(local);
+	}	
+	
 	wxString site;
 	if (pCommandLine->HasSwitch(CCommandLine::sitemanager))
 	{
-		Show();
-		OpenSiteManager();
+		if (!COptions::Get()->GetOptionVal(OPTION_INTERFACE_SITEMANAGER_ON_STARTUP) != 0)
+		{
+			Show();
+			OpenSiteManager();
+		}
 	}
 	else if ((site = pCommandLine->GetOption(CCommandLine::site)) != _T(""))
 	{
@@ -2442,12 +2527,7 @@ void CMainFrame::ProcessCommandLine()
 
 		if (pData)
 		{
-            if (ConnectToSite(pData))
-			{
-				SetBookmarksFromPath(site);
-				if (m_pMenuBar)
-					m_pMenuBar->UpdateBookmarkMenu();
-			}
+			ConnectToSite(pData);
 			delete pData;
 		}
 	}
@@ -2531,9 +2611,9 @@ WXLRESULT CMainFrame::MSWWindowProc(WXUINT nMsg, WXWPARAM wParam, WXLPARAM lPara
 		// create another instance of the module and call its Exit() member.
 		// After that, the next call to a wxDisplay will create a new factory and
 		// get the new display layout from Windows.
-		// 
+		//
 		// Note: Both the factory pattern as well as the dynamic object system
-		//       are perfect example of bad design.
+		//	   are perfect example of bad design.
 		//
 		wxModule* pDisplayModule = (wxModule*)wxCreateDynamicObject(_T("wxDisplayModule"));
 		if (pDisplayModule)
@@ -2653,6 +2733,12 @@ void CMainFrame::PostInitialize()
 	else
 		m_pUpdateWizard = 0;
 #endif //FZ_MANUALUPDATECHECK && FZ_AUTOUPDATECHECK
+
+	if (COptions::Get()->GetOptionVal(OPTION_INTERFACE_SITEMANAGER_ON_STARTUP) != 0)
+	{
+		Show();
+		OpenSiteManager();
+	}
 }
 
 void CMainFrame::OnMenuNewTab(wxCommandEvent& event)
@@ -2700,7 +2786,7 @@ void CMainFrame::SetBookmarksFromPath(const wxString& path)
 	}
 }
 
-bool CMainFrame::ConnectToServer(const CServer &server, const CServerPath &path /*=CServerPath()*/)
+bool CMainFrame::ConnectToServer(const CServer &server, const CServerPath &path /*=CServerPath()*/, bool isReconnect /*=true*/)
 {
 	CState* pState = CContextManager::Get()->GetCurrentContext();
 	if (!pState)
@@ -2730,6 +2816,10 @@ bool CMainFrame::ConnectToServer(const CServer &server, const CServerPath &path
 		}
 	}
 
+	CContextControl::_context_controls* controls = m_pContextControl->GetControlsFromState(pState);
+	if (!isReconnect && controls)
+		controls->site_bookmarks.clear();
+
 	return pState->Connect(server, path);
 }
 
@@ -2741,6 +2831,8 @@ void CMainFrame::OnToggleToolBar(wxCommandEvent& event)
 		m_pToolBar->Show( event.IsChecked() );
 #else
 	CreateToolBar();
+	if (m_pToolBar)
+		m_pToolBar->UpdateToolbarState();
 	HandleResize();
 #endif
 }
