@@ -55,14 +55,15 @@ public:
 	void ProcessCommandLine();
 
 	void PostInitialize();
-	
-	bool ConnectToServer(const CServer& server, const CServerPath& path = CServerPath());
+
+	bool ConnectToServer(const CServer& server, const CServerPath& path = CServerPath(), bool isReconnect = false);
 
 	CContextControl* GetContextControl() { return m_pContextControl; }
 
-	bool ConnectToSite(CSiteManagerItemData_Site* const pData);
+	bool ConnectToSite(CSiteManagerItemData_Site* const pData, bool newTab = false);
 
 protected:
+	bool CloseDialogsAndQuit(wxCloseEvent &event);
 	bool CreateMenus();
 	bool CreateQuickconnectBar();
 	bool CreateToolBar();
@@ -108,6 +109,7 @@ protected:
 	void OnSize(wxSizeEvent& event);
 	void OnMenuHandler(wxCommandEvent& event);
 	void OnEngineEvent(wxEvent& event);
+	void OnUpdateLedTooltip(wxCommandEvent& event);
 	void OnDisconnect(wxCommandEvent& event);
 	void OnCancel(wxCommandEvent& event);
 	void OnClose(wxCloseEvent& event);
