@@ -1,15 +1,19 @@
 #ifndef __STATUSVIEW_H__
 #define __STATUSVIEW_H__
 
+#ifdef __WXMSW__
+#include "richedit.h"
+#endif
+
 class CFastTextCtrl;
-class CStatusView :	public wxWindow
+class CStatusView : public wxWindow
 {
 public:
 	CStatusView(wxWindow* parent, wxWindowID id);
 	virtual ~CStatusView();
 
 	void AddToLog(CLogmsgNotification *pNotification);
-	void AddToLog(enum MessageType messagetype, wxString message);
+	void AddToLog(enum MessageType messagetype, const wxString& message, const wxDateTime& time);
 
 	void InitDefAttr();
 
@@ -25,7 +29,7 @@ protected:
 
 	void OnSize(wxSizeEvent &event);
 
-	DECLARE_EVENT_TABLE();
+	DECLARE_EVENT_TABLE()
 	void OnContextMenu(wxContextMenuEvent& event);
 	void OnClear(wxCommandEvent& event);
 	void OnCopy(wxCommandEvent& event);
@@ -37,10 +41,13 @@ protected:
 		wxString prefix;
 		int len;
 		wxTextAttr attr;
+#ifdef __WXMSW__
+		CHARFORMAT2 cf;
+#endif
 	} m_attributeCache[MessageTypeCount];
 
 	bool m_rtl;
-	
+
 	bool m_shown;
 
 	// Don't update actual log window if not shown,
@@ -49,8 +56,17 @@ protected:
 	{
 		MessageType messagetype;
 		wxString message;
+		wxDateTime time;
 	};
 	std::list<t_line> m_hiddenLines;
+
+	bool m_showTimestamps;
+	wxDateTime m_lastTime;
+	wxString m_lastTimeString;
+
+#ifdef __WXMAC__
+	int m_insertionPoint;
+#endif
 };
 
 #endif
