@@ -8,12 +8,12 @@
 #include "local_filesys.h"
 
 // Defined in optionspage_edit.cpp
-bool UnquoteCommand(wxString& command, wxString& arguments);
+bool UnquoteCommand(wxString& command, wxString& arguments, bool is_dde = false);
 bool ProgramExists(const wxString& editor);
 
 class CChangedFileDialog : public wxDialogEx
 {
-	DECLARE_EVENT_TABLE();
+	DECLARE_EVENT_TABLE()
 	void OnYes(wxCommandEvent& event);
 	void OnNo(wxCommandEvent& event);
 };
@@ -205,6 +205,8 @@ void CEditHandler::Release()
 			close(m_lockfile_descriptor);
 #endif
 
+		wxRemoveFile(m_localDir + _T("empty_file_yq744zm"));
+
 		RemoveAll(true);
 		wxRmdir(m_localDir);
 	}
@@ -246,7 +248,7 @@ int CEditHandler::GetFileCount(enum CEditHandler::fileType type, enum CEditHandl
 	{
 		if (type != remote)
 		{
-			for (std::list<t_fileData>::const_iterator iter = m_fileDataList[local].begin(); iter != m_fileDataList[local].end(); iter++)
+			for (std::list<t_fileData>::const_iterator iter = m_fileDataList[local].begin(); iter != m_fileDataList[local].end(); ++iter)
 			{
 				if (iter->state != state)
 					continue;
@@ -257,7 +259,7 @@ int CEditHandler::GetFileCount(enum CEditHandler::fileType type, enum CEditHandl
 		}
 		if (type != local)
 		{
-			for (std::list<t_fileData>::const_iterator iter = m_fileDataList[remote].begin(); iter != m_fileDataList[remote].end(); iter++)
+			for (std::list<t_fileData>::const_iterator iter = m_fileDataList[remote].begin(); iter != m_fileDataList[remote].end(); ++iter)
 			{
 				if (iter->state != state)
 					continue;
@@ -355,7 +357,7 @@ bool CEditHandler::RemoveAll(bool force)
 {
 	std::list<t_fileData> keep;
 
-	for (std::list<t_fileData>::iterator iter = m_fileDataList[remote].begin(); iter != m_fileDataList[remote].end(); iter++)
+	for (std::list<t_fileData>::iterator iter = m_fileDataList[remote].begin(); iter != m_fileDataList[remote].end(); ++iter)
 	{
 		if (!force && (iter->state == download || iter->state == upload || iter->state == upload_and_remove))
 		{
@@ -376,7 +378,7 @@ bool CEditHandler::RemoveAll(bool force)
 	m_fileDataList[remote].swap(keep);
 	keep.clear();
 
-	for (std::list<t_fileData>::iterator iter = m_fileDataList[local].begin(); iter != m_fileDataList[local].end(); iter++)
+	for (std::list<t_fileData>::iterator iter = m_fileDataList[local].begin(); iter != m_fileDataList[local].end(); ++iter)
 	{
 		if (force)
 			continue;
@@ -401,7 +403,7 @@ bool CEditHandler::RemoveAll(enum fileState state, const CServer* pServer /*=0*/
 
 	std::list<t_fileData> keep;
 
-	for (std::list<t_fileData>::iterator iter = m_fileDataList[remote].begin(); iter != m_fileDataList[remote].end(); iter++)
+	for (std::list<t_fileData>::iterator iter = m_fileDataList[remote].begin(); iter != m_fileDataList[remote].end(); ++iter)
 	{
 		if (iter->state != state)
 		{
@@ -433,7 +435,7 @@ bool CEditHandler::RemoveAll(enum fileState state, const CServer* pServer /*=0*/
 std::list<CEditHandler::t_fileData>::iterator CEditHandler::GetFile(const wxString& fileName)
 {
 	std::list<t_fileData>::iterator iter;
-	for (iter = m_fileDataList[local].begin(); iter != m_fileDataList[local].end(); iter++)
+	for (iter = m_fileDataList[local].begin(); iter != m_fileDataList[local].end(); ++iter)
 	{
 		if (iter->file == fileName)
 			break;
@@ -445,7 +447,7 @@ std::list<CEditHandler::t_fileData>::iterator CEditHandler::GetFile(const wxStri
 std::list<CEditHandler::t_fileData>::const_iterator CEditHandler::GetFile(const wxString& fileName) const
 {
 	std::list<t_fileData>::const_iterator iter;
-	for (iter = m_fileDataList[local].begin(); iter != m_fileDataList[local].end(); iter++)
+	for (iter = m_fileDataList[local].begin(); iter != m_fileDataList[local].end(); ++iter)
 	{
 		if (iter->file == fileName)
 			break;
@@ -457,7 +459,7 @@ std::list<CEditHandler::t_fileData>::const_iterator CEditHandler::GetFile(const
 std::list<CEditHandler::t_fileData>::iterator CEditHandler::GetFile(const wxString& fileName, const CServerPath& remotePath, const CServer& server)
 {
 	std::list<t_fileData>::iterator iter;
-	for (iter = m_fileDataList[remote].begin(); iter != m_fileDataList[remote].end(); iter++)
+	for (iter = m_fileDataList[remote].begin(); iter != m_fileDataList[remote].end(); ++iter)
 	{
 		if (iter->name != fileName)
 			continue;
@@ -477,7 +479,7 @@ std::list<CEditHandler::t_fileData>::iterator CEditHandler::GetFile(const wxStri
 std::list<CEditHandler::t_fileData>::const_iterator CEditHandler::GetFile(const wxString& fileName, const CServerPath& remotePath, const CServer& server) const
 {
 	std::list<t_fileData>::const_iterator iter;
-	for (iter = m_fileDataList[remote].begin(); iter != m_fileDataList[remote].end(); iter++)
+	for (iter = m_fileDataList[remote].begin(); iter != m_fileDataList[remote].end(); ++iter)
 	{
 		if (iter->name != fileName)
 			continue;
@@ -594,14 +596,12 @@ bool CEditHandler::StartEditing(enum CEditHandler::fileType type, t_fileData& da
 	wxASSERT(type != none);
 	wxASSERT(data.state == edit);
 
-	wxFileName fn(data.file);
-
-	if (!fn.FileExists())
+	bool is_link;
+	if (CLocalFileSystem::GetFileInfo(data.file, is_link, 0, &data.modificationTime, 0) != CLocalFileSystem::file)
 		return false;
-	data.modificationTime = fn.GetModificationTime();
 
 	bool program_exists = false;
-	wxString cmd = GetOpenCommand(fn.GetFullPath(), program_exists);
+	wxString cmd = GetOpenCommand(data.file, program_exists);
 	if (cmd.empty() || !program_exists)
 		return false;
 
@@ -629,27 +629,20 @@ void CEditHandler::CheckForModifications(bool emitEvent)
 	for (int i = 0; i < 2; i++)
 	{
 checkmodifications_loopbegin:
-		for (std::list<t_fileData>::iterator iter = m_fileDataList[i].begin(); iter != m_fileDataList[i].end(); iter++)
+		for (std::list<t_fileData>::iterator iter = m_fileDataList[i].begin(); iter != m_fileDataList[i].end(); ++iter)
 		{
 			if (iter->state != edit)
 				continue;
 
-			wxFileName fn(iter->file);
-			if (!fn.FileExists())
-			{
+			wxDateTime mtime;
+			bool is_link;
+			if (CLocalFileSystem::GetFileInfo(iter->file, is_link, 0, &mtime, 0) != CLocalFileSystem::file) {
 				m_fileDataList[i].erase(iter);
 
 				// Evil goto. Imo the next C++ standard needs a comefrom keyword.
 				goto checkmodifications_loopbegin;
 			}
 
-			wxDateTime mtime;
-
-			{
-				wxLogNull log; // If GetModificationTime fails wx spams error messages
-				mtime = fn.GetModificationTime();
-			}
-
 			if (!mtime.IsValid())
 				continue;
 
@@ -689,7 +682,7 @@ checkmodifications_loopbegin:
 			{
 				if (i == remote)
 				{
-					if (!fn.FileExists() || wxRemoveFile(fn.GetFullPath()))
+					if (CLocalFileSystem::GetFileInfo(iter->file, is_link, 0, &mtime, 0) != CLocalFileSystem::file || wxRemoveFile(iter->file))
 					{
 						m_fileDataList[i].erase(iter);
 						goto checkmodifications_loopbegin;
@@ -702,7 +695,7 @@ checkmodifications_loopbegin:
 					goto checkmodifications_loopbegin;
 				}
 			}
-			else if (!fn.FileExists())
+			else if (CLocalFileSystem::GetFileInfo(iter->file, is_link, 0, &mtime, 0) != CLocalFileSystem::file)
 			{
 				m_fileDataList[i].erase(iter);
 				goto checkmodifications_loopbegin;
@@ -740,7 +733,7 @@ int CEditHandler::DisplayChangeNotification(CEditHandler::fileType type, std::li
 		int pos = file.Find(wxFileName::GetPathSeparator(), true);
 		wxASSERT(pos != -1);
 		file = file.Mid(pos + 1);
-	
+
 		if (file == iter->name)
 		{
 			XRCCTRL(dlg, "ID_DESC_OPENEDAS", wxStaticText)->Hide();
@@ -811,7 +804,7 @@ bool CEditHandler::UploadFile(enum fileType type, std::list<t_fileData>::iterato
 		return false;
 	}
 
-	m_pQueue->QueueFile(false, false, localPath, file, iter->name, iter->remotePath, iter->server, wxLongLong(size.GetHi(), size.GetLo()), type);
+	m_pQueue->QueueFile(false, false, file, iter->name, localPath, iter->remotePath, iter->server, wxLongLong(size.GetHi(), size.GetLo()), type, priority_high);
 	m_pQueue->QueueFile_Finish(true);
 
 	return true;
@@ -963,7 +956,7 @@ wxString CEditHandler::GetSystemOpenCommand(wxString file, bool &program_exists)
 	if (ext == _T(""))
 		return _T("");
 
-	while (true)
+	for (;;)
 	{
 		wxFileType* pType = wxTheMimeTypesManager->GetFileTypeFromExtension(ext);
 		if (!pType)
@@ -983,6 +976,8 @@ wxString CEditHandler::GetSystemOpenCommand(wxString file, bool &program_exists)
 		program_exists = false;
 
 		wxString editor;
+		bool is_dde = false;
+#ifdef __WXMSW__
 		if (cmd.Left(7) == _T("WX_DDE#"))
 		{
 			// See wxWidget's wxExecute in src/msw/utilsexc.cpp
@@ -992,12 +987,16 @@ wxString CEditHandler::GetSystemOpenCommand(wxString file, bool &program_exists)
 			if (pos < 1)
 				return cmd;
 			editor = editor.Left(pos);
+			is_dde = true;
 		}
 		else
+#endif
+		{
 			editor = cmd;
+		}
 
 		wxString args;
-		if (!UnquoteCommand(editor, args) || editor.empty())
+		if (!UnquoteCommand(editor, args, is_dde) || editor.empty())
 			return cmd;
 
 		if (!PathExpand(editor))
@@ -1071,7 +1070,7 @@ wxString CEditHandler::GetCustomOpenCommand(const wxString& file, bool& program_
 			program_exists = false;
 			return prog;
 		}
-		
+
 		program_exists = true;
 		return command + _T(" \"") + fn.GetFullPath() + _T("\"");
 	}
@@ -1152,13 +1151,13 @@ wxString CEditHandler::TruncateFilename(const wxString path, const wxString& nam
 			return name.Left(max - pathlen - extlen) + name.Mid(pos);
 		}
 	}
-	
+
 	return name;
 }
 
 bool CEditHandler::FilenameExists(const wxString& file)
 {
-	for (std::list<t_fileData>::const_iterator iter = m_fileDataList[remote].begin(); iter != m_fileDataList[remote].end(); iter++)
+	for (std::list<t_fileData>::const_iterator iter = m_fileDataList[remote].begin(); iter != m_fileDataList[remote].end(); ++iter)
 	{
 #ifdef __WXMSW__
 		if (!iter->file.CmpNoCase(file))
@@ -1235,7 +1234,7 @@ int CEditHandlerStatusDialog::ShowModal()
 	{
 		const std::list<CEditHandler::t_fileData>& files = pEditHandler->GetFiles(CEditHandler::remote);
 		unsigned int i = 0;
-		for (std::list<CEditHandler::t_fileData>::const_iterator iter = files.begin(); iter != files.end(); iter++, i++)
+		for (std::list<CEditHandler::t_fileData>::const_iterator iter = files.begin(); iter != files.end(); ++iter, ++i)
 		{
 			pListCtrl->InsertItem(i, iter->name);
 			pListCtrl->SetItem(i, COLUMN_TYPE, _("Remote"));
@@ -1272,7 +1271,7 @@ int CEditHandlerStatusDialog::ShowModal()
 	{
 		const std::list<CEditHandler::t_fileData>& files = pEditHandler->GetFiles(CEditHandler::local);
 		unsigned int i = 0;
-		for (std::list<CEditHandler::t_fileData>::const_iterator iter = files.begin(); iter != files.end(); iter++, i++)
+		for (std::list<CEditHandler::t_fileData>::const_iterator iter = files.begin(); iter != files.end(); ++iter, ++i)
 		{
 			pListCtrl->InsertItem(i, iter->file);
 			pListCtrl->SetItem(i, COLUMN_TYPE, _("Local"));
@@ -1376,13 +1375,13 @@ void CEditHandlerStatusDialog::OnUnedit(wxCommandEvent& event)
 		files.push_front(item);
 	}
 
-	for (std::list<int>::const_iterator iter = files.begin(); iter != files.end(); iter++)
+	for (std::list<int>::const_iterator iter = files.begin(); iter != files.end(); ++iter)
 	{
 		const int i = *iter;
 
 		enum CEditHandler::fileType type;
 		CEditHandler::t_fileData* pData = GetDataFromItem(i, type);
-		
+
 		if (type == CEditHandler::local)
 		{
 			pEditHandler->Remove(pData->file);
@@ -1429,7 +1428,7 @@ void CEditHandlerStatusDialog::OnUpload(wxCommandEvent& event)
 		files.push_front(item);
 	}
 
-	for (std::list<int>::const_iterator iter = files.begin(); iter != files.end(); iter++)
+	for (std::list<int>::const_iterator iter = files.begin(); iter != files.end(); ++iter)
 	{
 		const int i = *iter;
 
@@ -1467,7 +1466,7 @@ void CEditHandlerStatusDialog::OnEdit(wxCommandEvent& event)
 	while ((item = pListCtrl->GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1)
 	{
 		pListCtrl->SetItemState(item, 0, wxLIST_STATE_SELECTED);
-		
+
 		enum CEditHandler::fileType type;
 		CEditHandler::t_fileData* pData = GetDataFromItem(item, type);
 
@@ -1479,7 +1478,7 @@ void CEditHandlerStatusDialog::OnEdit(wxCommandEvent& event)
 		files.push_front(item);
 	}
 
-	for (std::list<int>::const_iterator iter = files.begin(); iter != files.end(); iter++)
+	for (std::list<int>::const_iterator iter = files.begin(); iter != files.end(); ++iter)
 	{
 		const int i = *iter;
 
@@ -1559,7 +1558,7 @@ bool CNewAssociationDialog::Show(const wxString &file)
 	if (!pos)
 		m_ext = _T(".");
 	else if (pos != -1)
-		m_ext = file.Mid(pos + 1);	
+		m_ext = file.Mid(pos + 1);
 	else
 		m_ext.clear();
 
