@@ -9,10 +9,10 @@
 #include "commctrl.h"
 
 #ifndef HDF_SORTUP
-#define HDF_SORTUP              0x0400
+#define HDF_SORTUP				0x0400
 #endif
 #ifndef HDF_SORTDOWN
-#define HDF_SORTDOWN            0x0200
+#define HDF_SORTDOWN			0x0200
 #endif
 #else
 #include "themeprovider.h"
@@ -71,7 +71,7 @@ wxListCtrlEx::~wxListCtrlEx()
 }
 
 #ifndef __WXMSW__
-wxScrolledWindow* wxListCtrlEx::GetMainWindow()
+wxScrolledWindow* wxListCtrlEx::GetMainWindow() const
 {
 #ifdef __WXMAC__
 	return (wxScrolledWindow*)m_genericImpl->m_mainWin;
@@ -285,7 +285,7 @@ void wxListCtrlEx::OnKeyDown(wxKeyEvent& event)
 		return;
 	}
 
-#if defined(__WXMSW__) && wxUSE_UNICODE
+#if defined(__WXMSW__)
 
 	if (code >= 300 && code != WXK_NUMPAD_DECIMAL)
 	{
@@ -303,7 +303,7 @@ void wxListCtrlEx::OnKeyDown(wxKeyEvent& event)
 		event.Skip();
 		return;
 	}
-	
+
 	key = buffer[0];
 
 	if (key < 32)
@@ -321,16 +321,14 @@ void wxListCtrlEx::OnKeyDown(wxKeyEvent& event)
 #else
 	if (code > 32 && code < 300 && !event.HasModifiers())
 	{
-#if wxUSE_UNICODE
 		int unicodeKey = event.GetUnicodeKey();
 		if (unicodeKey)
 			code = unicodeKey;
-#endif
 		HandlePrefixSearch(code);
 	}
 	else
 		event.Skip();
-#endif //defined(__WXMSW__) && wxUSE_UNICODE
+#endif //defined(__WXMSW__)
 }
 
 // Declared const due to design error in wxWidgets.
@@ -626,11 +624,11 @@ void wxListCtrlEx::MoveColumn(unsigned int col, unsigned int before)
 			info.order++;
 	}
 
-	int icon = -1;
-
 	t_columnInfo& info = m_columnInfo[col];
+
 	if (info.shown)
 	{
+		int icon = -1;
 		// Remove old column
 		for (unsigned int i = 0; i < (unsigned int)GetColumnCount(); i++)
 		{
@@ -1112,7 +1110,7 @@ bool wxListCtrlEx::MSWOnNotify(int idCtrl, WXLPARAM lParam, WXLPARAM *result)
 	NMHDR *nmhdr = (NMHDR *)lParam;
 	HWND hwndHdr = ListView_GetHeader((HWND)GetHandle());
 
-    if (nmhdr->hwndFrom != hwndHdr)
+	if (nmhdr->hwndFrom != hwndHdr)
 		return wxListCtrl::MSWOnNotify(idCtrl, lParam, result);
 
 	HD_NOTIFY *nmHDR = (HD_NOTIFY *)nmhdr;
@@ -1143,7 +1141,7 @@ bool wxListCtrlEx::MSWOnNotify(int idCtrl, WXLPARAM lParam, WXLPARAM *result)
 				return false;
 			}
 		}
-		else 
+		else
 			return false;
 	}
 
@@ -1160,3 +1158,163 @@ bool wxListCtrlEx::HasSelection() const
 	return GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED) != -1;
 #endif
 }
+
+
+wxRect wxListCtrlEx::GetListRect() const
+{
+#ifdef __WXMSW__
+	wxRect windowRect = GetClientRect();
+	{
+		wxRect topRect;
+		if (GetItemRect(0, topRect))
+		{
+			windowRect.height -= topRect.y;
+			windowRect.y += topRect.y;
+		}
+	}
+#else
+	wxRect windowRect = GetMainWindow()->GetClientRect();
+#endif
+
+	return windowRect;
+}
+
+
+CListCtrlDropTarget::CListCtrlDropTarget(wxListCtrlEx* pListCtrl)
+	: m_pListCtrl(pListCtrl)
+	, m_count()
+{
+	m_timer.SetOwner(this);
+}
+
+
+bool CListCtrlDropTarget::OnDrop(wxCoord x, wxCoord y)
+{
+	m_timer.Stop();
+	return true;
+}
+
+
+wxDragResult CListCtrlDropTarget::OnDragOver(wxCoord x, wxCoord y, wxDragResult def)
+{
+	if (!m_timer.IsRunning() && IsScroll(wxPoint(x, y)))
+	{
+		m_timer.Start(100, true);
+		m_count = 0;
+	}
+	return def;
+}
+
+
+
+void CListCtrlDropTarget::OnLeave()
+{
+	m_timer.Stop();
+}
+
+
+wxDragResult CListCtrlDropTarget::OnEnter(wxCoord x, wxCoord y, wxDragResult def)
+{
+	if (!m_timer.IsRunning() && IsScroll(wxPoint(x, y)))
+	{
+		m_timer.Start(100, true);
+		m_count = 0;
+	}
+	return def;
+}
+
+
+bool CListCtrlDropTarget::IsScroll(wxPoint p) const
+{
+	return IsTopScroll(p) || IsBottomScroll(p);
+}
+
+
+bool CListCtrlDropTarget::IsTopScroll(wxPoint p) const
+{
+	wxRect itemRect;
+	if (!m_pListCtrl->GetItemRect(m_pListCtrl->GetTopItem(), itemRect))
+		return false;
+
+	wxRect windowRect = m_pListCtrl->GetListRect();
+
+	if (itemRect.GetHeight() > windowRect.GetHeight() / 4 ) {
+		itemRect.SetHeight( wxMax( windowRect.GetHeight() / 4, 8 ) );
+	}
+
+	if (p.y < 0 || p.y >= itemRect.GetBottom())
+		return false;
+
+	if (p.x < 0 || p.x > windowRect.GetWidth())
+		return false;
+
+	int top = m_pListCtrl->GetTopItem();
+	if( top <= 0 ) {
+		return false;
+	}
+
+	return true;
+}
+
+
+bool CListCtrlDropTarget::IsBottomScroll(wxPoint p) const
+{
+	wxRect itemRect;
+	if (!m_pListCtrl->GetItemRect(0, itemRect))
+		return false;
+
+	wxRect windowRect = m_pListCtrl->GetListRect();
+	if (itemRect.GetHeight() > windowRect.GetHeight() / 4 ) {
+		itemRect.SetHeight( wxMax( windowRect.GetHeight() / 4, 8 ) );
+	}
+
+	if (p.y > windowRect.GetBottom() || p.y < windowRect.GetBottom() - itemRect.GetHeight())
+		return false;
+
+	if (p.x < 0 || p.x > windowRect.GetWidth())
+		return false;
+
+	int bottom = m_pListCtrl->GetTopItem() + m_pListCtrl->GetCountPerPage();
+	if (bottom >= m_pListCtrl->GetItemCount())
+		return false;
+
+	return true;
+}
+
+
+void CListCtrlDropTarget::OnTimer(wxTimerEvent& /*event*/)
+{
+	wxPoint p = wxGetMousePosition();
+#ifdef __WXMSW__
+	wxWindow* ctrl = m_pListCtrl;
+#else
+	wxWindow* ctrl = m_pListCtrl->GetMainWindow();
+#endif
+	p = ctrl->ScreenToClient(p);
+
+	if (IsTopScroll(p))
+	{
+		int top = m_pListCtrl->GetTopItem();
+		m_pListCtrl->EnsureVisible(top - 1);
+	}
+	else if (IsBottomScroll(p))
+	{
+		int top = m_pListCtrl->GetTopItem();
+		m_pListCtrl->EnsureVisible(top + m_pListCtrl->GetCountPerPage());
+	}
+	else
+	{
+		m_timer.Stop();
+		return;
+	}
+
+	DisplayDropHighlight(p);
+
+	if (m_count < 90)
+		++m_count;
+	m_timer.Start(100 - m_count, true);
+}
+
+BEGIN_EVENT_TABLE(CListCtrlDropTarget, wxEvtHandler)
+EVT_TIMER(wxID_ANY, CListCtrlDropTarget::OnTimer)
+END_EVENT_TABLE()
