@@ -352,7 +352,7 @@ void CManualTransfer::OnOK(wxCommandEvent& event)
 	}
 
 	int old_data_type = COptions::Get()->GetOptionVal(OPTION_ASCIIBINARY);
-	
+
 	// Set data type for the file to add
 	if (XRCCTRL(*this, "ID_TYPE_ASCII", wxRadioButton)->GetValue())
 		COptions::Get()->SetOption(OPTION_ASCIIBINARY, 1);
@@ -370,7 +370,10 @@ void CManualTransfer::OnOK(wxCommandEvent& event)
 		return;
 	}
 
-	m_pQueueView->QueueFile(!start, download, localPath, name, remote_file, path, *m_pServer, -1);
+	m_pQueueView->QueueFile(!start, download,
+		download ? remote_file : name,
+		(remote_file != name) ? (download ? name : remote_file) : wxString(),
+		localPath, path, *m_pServer, -1);
 
 	// Restore old data type
 	COptions::Get()->SetOption(OPTION_ASCIIBINARY, old_data_type);
@@ -447,8 +450,17 @@ bool CManualTransfer::VerifyServer()
 		(logon_type == ACCOUNT || logon_type == NORMAL))
 	{
 		XRCCTRL(*this, "ID_LOGONTYPE", wxChoice)->SetFocus();
-		wxMessageBox(_("FileZilla is running in kiosk mode.\n'Normal' and 'Account' logontypes are not available in this mode."));
-		return false;
+		wxString msg;
+		if (COptions::Get()->OptionFromFzDefaultsXml(OPTION_DEFAULT_KIOSKMODE))
+			msg = _("Saving of password has been disabled by your system administrator.");
+		else
+			msg = _("Saving of passwords has been disabled by you.");
+		msg += _T("\n");
+		msg += _("'Normal' and 'Account' logontypes are not available, using 'Ask for password' instead.");
+		XRCCTRL(*this, "ID_LOGONTYPE", wxChoice)->SetStringSelection(CServer::GetNameFromLogonType(ASK));
+		XRCCTRL(*this, "ID_PASS", wxTextCtrl)->SetValue(_T(""));
+		logon_type = ASK;
+		wxMessageBox(msg, _("Cannot remember password"), wxICON_INFORMATION, this);
 	}
 
 	CServer server;
@@ -534,8 +546,7 @@ void CManualTransfer::OnSelectSite(wxCommandEvent& event)
 
 void CManualTransfer::OnSelectedSite(wxCommandEvent& event)
 {
-	wxString path;
-	CSiteManagerItemData_Site* pData = CSiteManager::GetSiteById(event.GetId(), path);
+	CSiteManagerItemData_Site* pData = CSiteManager::GetSiteById(event.GetId());
 	if (!pData)
 		return;
 
