@@ -6,7 +6,7 @@
 #include "Options.h"
 #include "xmlfunctions.h"
 
-std::map<int, struct CSiteManager::_menu_data> CSiteManager::m_idMap;
+std::map<int, CSiteManagerItemData_Site*> CSiteManager::m_idMap;
 
 bool CSiteManager::Load(TiXmlElement *pElement, CSiteManagerXmlHandler* pHandler)
 {
@@ -109,7 +109,7 @@ CSiteManagerItemData_Site* CSiteManager::ReadServerElement(TiXmlElement *pElemen
 class CSiteManagerXmlHandler_Menu : public CSiteManagerXmlHandler
 {
 public:
-	CSiteManagerXmlHandler_Menu(wxMenu* pMenu, std::map<int, struct CSiteManager::_menu_data> *idMap, bool predefined)
+	CSiteManagerXmlHandler_Menu(wxMenu* pMenu, std::map<int, CSiteManagerItemData_Site*> *idMap, bool predefined)
 		: m_pMenu(pMenu), m_idMap(idMap)
 	{
 		m_added_site = false;
@@ -147,11 +147,7 @@ public:
 		m_parents.push_back(m_pMenu);
 		m_childNames.push_back(name);
 		m_paths.push_back(path);
-
-		wxString str(name);
-		str.Replace(_T("\\"), _T("\\\\"));
-		str.Replace(_T("/"), _T("\\/"));
-		path += _T("/") + str;
+		path += _T("/") + CSiteManager::EscapeSegment(name);
 
 		m_pMenu = new wxMenu;
 
@@ -165,14 +161,9 @@ public:
 		newName.Replace(_T("&"), _T("&&"));
 		wxMenuItem* pItem = m_pMenu->Insert(i, wxID_ANY, newName);
 
-		struct CSiteManager::_menu_data menu_data;
-		menu_data.data = data;
-		menu_data.path = data->m_server.GetName();
-		menu_data.path.Replace(_T("\\"), _T("\\\\"));
-		menu_data.path.Replace(_T("/"), _T("\\/"));
-		menu_data.path = path + _T("/") + menu_data.path;
+		data->m_path = path + _T("/") + CSiteManager::EscapeSegment(data->m_server.GetName());
 
-		(*m_idMap)[pItem->GetId()] = menu_data;
+		(*m_idMap)[pItem->GetId()] = data;
 
 		m_added_site = true;
 
@@ -222,7 +213,7 @@ public:
 protected:
 	wxMenu* m_pMenu;
 
-	std::map<int, struct CSiteManager::_menu_data> *m_idMap;
+	std::map<int, CSiteManagerItemData_Site*> *m_idMap;
 
 	std::list<wxMenu*> m_parents;
 	std::list<wxString> m_childNames;
@@ -303,13 +294,13 @@ wxMenu* CSiteManager::GetSitesMenu()
 
 void CSiteManager::ClearIdMap()
 {
-	for (std::map<int, struct _menu_data>::iterator iter = m_idMap.begin(); iter != m_idMap.end(); ++iter)
-		delete iter->second.data;
+	for (std::map<int, CSiteManagerItemData_Site*>::iterator iter = m_idMap.begin(); iter != m_idMap.end(); ++iter)
+		delete iter->second;
 
 	m_idMap.clear();
 }
 
-wxMenu* CSiteManager::GetSitesMenu_Predefined(std::map<int, struct _menu_data> &idMap)
+wxMenu* CSiteManager::GetSitesMenu_Predefined(std::map<int, CSiteManagerItemData_Site*> &idMap)
 {
 	const wxString& defaultsDir = wxGetApp().GetDefaultsDir();
 	if (defaultsDir == _T(""))
@@ -344,16 +335,15 @@ wxMenu* CSiteManager::GetSitesMenu_Predefined(std::map<int, struct _menu_data> &
 	return pMenu;
 }
 
-CSiteManagerItemData_Site* CSiteManager::GetSiteById(int id, wxString& path)
+CSiteManagerItemData_Site* CSiteManager::GetSiteById(int id)
 {
-	std::map<int, struct _menu_data>::iterator iter = m_idMap.find(id);
+	std::map<int, CSiteManagerItemData_Site*>::iterator iter = m_idMap.find(id);
 
 	CSiteManagerItemData_Site *pData;
 	if (iter != m_idMap.end())
 	{
-		pData = iter->second.data;
-		iter->second.data = 0;
-		path = iter->second.path;
+		pData = iter->second;
+		iter->second = 0;
 	}
 	else
 		pData = 0;
@@ -411,6 +401,22 @@ bool CSiteManager::UnescapeSitePath(wxString path, std::list<wxString>& result)
 	return !result.empty();
 }
 
+wxString CSiteManager::EscapeSegment(wxString segment)
+{
+	segment.Replace(_T("\\"), _T("\\\\"));
+	segment.Replace(_T("/"), _T("\\/"));
+	return segment;
+}
+
+wxString CSiteManager::BuildPath(wxChar root, std::list<wxString> const& segments)
+{
+	wxString ret = root;
+	for (std::list<wxString>::const_iterator it = segments.begin(); it != segments.end(); ++it)
+		ret += _T("/") + EscapeSegment(*it);
+
+	return ret;
+}
+
 CSiteManagerItemData_Site* CSiteManager::GetSiteByPath(wxString sitePath)
 {
 	wxChar c = sitePath[0];
@@ -458,7 +464,7 @@ CSiteManagerItemData_Site* CSiteManager::GetSiteByPath(wxString sitePath)
 	}
 
 	std::list<wxString> segments;
-	if (!UnescapeSitePath(sitePath, segments))
+	if (!UnescapeSitePath(sitePath, segments) || segments.empty())
 	{
 		wxMessageBox(_("Site path is malformed."), _("Invalid site path"));
 		return 0;
@@ -476,6 +482,7 @@ CSiteManagerItemData_Site* CSiteManager::GetSiteByPath(wxString sitePath)
 	{
 		pBookmark = pChild;
 		pChild = pChild->Parent()->ToElement();
+		segments.pop_back();
 	}
 	else
 		pBookmark = 0;
@@ -511,6 +518,8 @@ CSiteManagerItemData_Site* CSiteManager::GetSiteByPath(wxString sitePath)
 		data->m_remoteDir = remotePath;
 	}
 
+	data->m_path = BuildPath( c, segments );
+
 	return data;
 }
 
@@ -558,6 +567,14 @@ bool CSiteManager::GetBookmarks(wxString sitePath, std::list<wxString> &bookmark
 	}
 
 	TiXmlElement* pChild = GetElementByPath(pElement, segments);
+
+	TiXmlElement* pBookmark = 0;
+	if (pChild && !strcmp(pChild->Value(), "Bookmark"))
+	{
+		pBookmark = pChild;
+		pChild = pChild->Parent()->ToElement();
+	}
+
 	if (!pChild || strcmp(pChild->Value(), "Server"))
 		return 0;
 
@@ -582,7 +599,7 @@ bool CSiteManager::GetBookmarks(wxString sitePath, std::list<wxString> &bookmark
 		if (localPath.empty() && remotePath.IsEmpty())
 			continue;
 
-		bookmarks.push_back(name);		
+		bookmarks.push_back(name);
 	}
 
 	return true;
@@ -622,7 +639,7 @@ wxString CSiteManager::AddServer(CServer server)
 	wxString name = _("New site");
 	int i = 1;
 
-	while (true)
+	for (;;)
 	{
 		std::list<wxString>::const_iterator iter;
 		for (iter = names.begin(); iter != names.end(); ++iter)
@@ -658,20 +675,16 @@ wxString CSiteManager::AddServer(CServer server)
 		wxMessageBox(msg, _("Error writing xml file"), wxICON_ERROR);
 		return _T("");
 	}
-	
-	name.Replace(_T("\\"), _T("\\\\"));
-	name.Replace(_T("/"), _T("\\/"));
 
-	return _T("0/") + name;
+	return _T("0/") + EscapeSegment(name);
 }
 
-TiXmlElement* CSiteManager::GetElementByPath(TiXmlElement* pNode, std::list<wxString> &segments)
+TiXmlElement* CSiteManager::GetElementByPath(TiXmlElement* pNode, std::list<wxString> const& segments)
 {
-	while (!segments.empty())
+	for (std::list<wxString>::const_iterator it = segments.begin(); it != segments.end(); ++it)
 	{
-		const wxString segment = segments.front();
-		segments.pop_front();
-
+		const wxString & segment = *it;
+		
 		TiXmlElement* pChild;
 		for (pChild = pNode->FirstChildElement(); pChild; pChild = pChild->NextSiblingElement())
 		{
@@ -713,9 +726,7 @@ bool CSiteManager::AddBookmark(wxString sitePath, const wxString& name, const wx
 	CInterProcessMutex mutex(MUTEX_SITEMANAGER);
 
 	CXmlFile file;
-	TiXmlElement* pDocument = 0;
-
-	pDocument = file.Load(_T("sitemanager"));
+	TiXmlElement* pDocument = file.Load(_T("sitemanager"));
 
 	if (!pDocument)
 	{
@@ -800,9 +811,7 @@ bool CSiteManager::ClearBookmarks(wxString sitePath)
 	CInterProcessMutex mutex(MUTEX_SITEMANAGER);
 
 	CXmlFile file;
-	TiXmlElement* pDocument = 0;
-
-	pDocument = file.Load(_T("sitemanager"));
+	TiXmlElement* pDocument = file.Load(_T("sitemanager"));
 
 	if (!pDocument)
 	{
