@@ -12,12 +12,10 @@
 bool CWrapEngine::m_use_cache = true;
 
 #define WRAPDEBUG 0
-#if wxUSE_UNICODE
 // Chinese equivalents to ".", "," and ":"
 static const wxChar noWrapChars_Chinese[] = { '.', ',', ':', 0x3002, 0xFF0C, 0xFF1A, 0};
 // Remark: Chinese (Taiwan) uses ascii punctuation marks though, but those
 // don't have to be added, as only characters >= 128 will be wrapped.
-#endif
 
 bool CWrapEngine::CanWrapBefore(const wxChar& c)
 {
@@ -199,16 +197,17 @@ bool CWrapEngine::WrapText(wxWindow* parent, wxString& text, unsigned long maxLe
 	- Wrap embedded English text fragments only on spaces
 
 	For this kind of languages, a different wrapping algorithm is used.
-    */
+	*/
 
 	if (!m_font.IsOk())
 		m_font = parent->GetFont();
 
+#ifdef __WXDEBUG__
+	const wxString original = text;
+#endif
+
 	if (m_wrapOnEveryChar)
 	{
-#ifdef __WXDEBUG__
-		const wxString original = text;
-#endif
 		bool res = WrapTextChinese(parent, text, maxLength);
 #ifdef __WXDEBUG__
 		wxString unwrapped = UnwrapText(text);
@@ -230,11 +229,15 @@ bool CWrapEngine::WrapText(wxWindow* parent, wxString& text, unsigned long maxLe
 	unsigned int lineLength = 0;
 
 	bool url = false;
+	bool containsURL = false;
 	for (int i = 0; i <= strLen; i++)
 	{
 		if ((text[i] == ':' && text[i + 1] == '/' && text[i + 2] == '/') || // absolute
 			(text[i] == '/' && (!i || text[i - 1] == ' '))) // relative
+		{
 			url = true;
+			containsURL = true;
+		}
 		if (text[i] != ' ' && text[i] != 0)
 		{
 			// If url, wrap on slashes and ampersands, but not first slash of something://
@@ -265,16 +268,23 @@ bool CWrapEngine::WrapText(wxWindow* parent, wxString& text, unsigned long maxLe
 
 		if (lineLength + spaceWidth + width > maxLength)
 		{
+			// Cannot be appended to current line without overflow, so start a new line
 			if (wrappedText != _T(""))
 				wrappedText += _T("\n");
 			wrappedText += text.Mid(start, wrapAfter - start);
 			if (text[wrapAfter] != ' ' && text[wrapAfter] != '\0')
 				wrappedText += text[wrapAfter];
+
 			if (width + spaceWidth >= (int)maxLength)
 			{
-				if (wrappedText != _T(""))
-					wrappedText += _T("\n");
-				wrappedText += text.Mid(wrapAfter + 1, i - wrapAfter - 1);
+				// Current segment too big to even fit into a line just by itself
+
+				if( i != wrapAfter )
+				{
+					if (wrappedText != _T(""))
+						wrappedText += _T("\n");
+					wrappedText += text.Mid(wrapAfter + 1, i - wrapAfter - 1);
+				}
 
 				start = i + 1;
 				wrapAfter = -1;
@@ -318,6 +328,11 @@ bool CWrapEngine::WrapText(wxWindow* parent, wxString& text, unsigned long maxLe
 
 	text = wrappedText;
 
+#ifdef __WXDEBUG__
+		wxString unwrapped = UnwrapText(text);
+		wxASSERT(original == unwrapped || containsURL);
+#endif
+
 	return true;
 }
 
@@ -343,10 +358,10 @@ bool CWrapEngine::WrapText(wxWindow* parent, int id, unsigned long maxLength)
 int CWrapEngine::WrapRecursive(wxWindow* wnd, wxSizer* sizer, int max)
 {
 	// This function auto-wraps static texts.
-	
+
 #if WRAPDEBUG >= 3
 	static int level = 1;
-    plvl printf("Enter\n");
+	plvl printf("Enter with max = %d\n", max);
 #endif
 
 	if (max <= 0)
@@ -380,8 +395,8 @@ int CWrapEngine::WrapRecursive(wxWindow* wnd, wxSizer* sizer, int max)
 		wxASSERT(min.GetWidth() + rborder + lborder <= sizer->GetMinSize().GetWidth());
 
 		if (min.GetWidth() + item->GetPosition().x + lborder + rborder <= max)
-		    continue;
-			
+			continue;
+
 		wxWindow* window;
 		wxSizer* subSizer = 0;
 		if ((window = item->GetWindow()))
@@ -396,13 +411,13 @@ int CWrapEngine::WrapRecursive(wxWindow* wnd, wxSizer* sizer, int max)
 #endif
 				if (max - rect.GetLeft() - rborder - offset <= 0)
 					continue;
-					
+
 				wxString str = text->GetLabel();
 				if (!WrapText(text, str, max - wxMax(0, rect.GetLeft()) - rborder - offset))
 				{
 #if WRAPDEBUG >= 3
 					plvl printf("Leave: WrapText failed\n");
-#endif		
+#endif
 					return result | wrap_failed;
 				}
 				text->SetLabel(str);
@@ -438,6 +453,17 @@ int CWrapEngine::WrapRecursive(wxWindow* wnd, wxSizer* sizer, int max)
 				}
 				continue;
 			}
+
+			if (wxDynamicCast(window, wxCheckBox) || wxDynamicCast(window, wxRadioButton) || wxDynamicCast(window, wxChoice))
+			{
+#if WRAPDEBUG >= 3
+				plvl printf("Leave: WrapRecursive on unshrinkable controls failed\n");
+#endif
+				result |= wrap_failed;
+				return result;
+			}
+
+			// We assume here that all other oversized controls can scale
 		}
 		else if ((subSizer = item->GetSizer()))
 		{
@@ -468,7 +494,7 @@ int CWrapEngine::WrapRecursive(wxWindow* wnd, wxSizer* sizer, int max)
 			}
 		}
 	}
-	
+
 #if WRAPDEBUG >= 3
 	plvl printf("Leave: Success\n");
 #endif
@@ -489,7 +515,7 @@ void CWrapEngine::UnwrapRecursive_Wrapped(const std::list<int> &wrapped, std::ve
 	unsigned int i = 0;
 	for (std::list<int>::const_iterator iter = wrapped.begin();
 		iter != wrapped.end();
-		iter++)
+		++iter)
 	{
 		UnwrapRecursive(windows[i], windows[i]->GetSizer());
 		windows[i]->GetSizer()->Layout();
@@ -504,7 +530,7 @@ void CWrapEngine::UnwrapRecursive_Wrapped(const std::list<int> &wrapped, std::ve
 				continue;
 			}
 		}
-		
+
 		i++;
 	}
 }
@@ -514,18 +540,18 @@ bool CWrapEngine::WrapRecursive(std::vector<wxWindow*>& windows, double ratio, c
 	int maxWidth = GetWidthFromCache(name);
 	if (maxWidth)
 	{
-		for (std::vector<wxWindow*>::iterator iter = windows.begin(); iter != windows.end(); iter++)
+		for (std::vector<wxWindow*>::iterator iter = windows.begin(); iter != windows.end(); ++iter)
 		{
 			wxSizer* pSizer = (*iter)->GetSizer();
 			if (!pSizer)
 				continue;
 
 			pSizer->Layout();
-			
+
 #ifdef __WXMAC__
 			const int offset = 6;
 #elif defined(__WXGTK__)
-			const int offset = 4;
+			const int offset = 0;
 #else
 			const int offset = 0;
 #endif
@@ -537,7 +563,9 @@ bool CWrapEngine::WrapRecursive(std::vector<wxWindow*>& windows, double ratio, c
 			wxASSERT(!(res & wrap_failed));
 			pSizer->Layout();
 			pSizer->Fit(*iter);
+#ifdef __WXDEBUG__
 			wxSize size = pSizer->GetMinSize();
+#endif
 			wxASSERT(size.x <= maxWidth);
 		}
 		return true;
@@ -547,7 +575,7 @@ bool CWrapEngine::WrapRecursive(std::vector<wxWindow*>& windows, double ratio, c
 
 	wxSize size = minRequestedSize;
 
-	for (std::vector<wxWindow*>::iterator iter = windows.begin(); iter != windows.end(); iter++)
+	for (std::vector<wxWindow*>::iterator iter = windows.begin(); iter != windows.end(); ++iter)
 	{
 		wxSizer* pSizer = (*iter)->GetSizer();
 		if (!pSizer)
@@ -575,25 +603,25 @@ bool CWrapEngine::WrapRecursive(std::vector<wxWindow*>& windows, double ratio, c
 
 	double bestRatioDiff = currentRatio - ratio;
 	int bestWidth = max;
-	
-#if WRAPDEBUG > 0	
+
+#if WRAPDEBUG > 0
 	printf("Target ratio: %f\n", (float)ratio);
 	printf("Canvas: % 4d % 4d\n", canvas.x, canvas.y);
 	printf("Initial min and max: %d %d\n", min, max);
 #endif
 
-	while (true)
+	for (;;)
 	{
 		std::list<int> didwrap;
 
 		wxSize size = minRequestedSize;
-		for (std::vector<wxWindow*>::iterator iter = windows.begin(); iter != windows.end(); iter++)
+		for (std::vector<wxWindow*>::iterator iter = windows.begin(); iter != windows.end(); ++iter)
 		{
 			wxSizer* pSizer = (*iter)->GetSizer();
 #ifdef __WXMAC__
 			const int offset = 6;
 #elif defined(__WXGTK__)
-			const int offset = 4;
+			const int offset = 0;
 #else
 			const int offset = 0;
 #endif
@@ -610,7 +638,7 @@ bool CWrapEngine::WrapRecursive(std::vector<wxWindow*>& windows, double ratio, c
 		}
 
 #if WRAPDEBUG > 0
-	    printf("Current: % 4d % 4d   desiredWidth: %d, min: %d, max: %d\n", size.GetWidth(), size.GetHeight(), desiredWidth, min, max);
+		printf("Current: % 4d % 4d   desiredWidth: %d, min: %d, max: %d\n", size.GetWidth(), size.GetHeight(), desiredWidth, min, max);
 #endif
 		if (size.GetWidth() > desiredWidth)
 		{
@@ -632,9 +660,9 @@ bool CWrapEngine::WrapRecursive(std::vector<wxWindow*>& windows, double ratio, c
 		actualWidth = size.GetWidth();
 
 		double newRatio = ((double)(size.GetWidth() + canvas.x) / (size.GetHeight() + canvas.y));
-#if WRAPDEBUG > 0		
+#if WRAPDEBUG > 0
 		printf("Ratio: %f\n", (float)newRatio);
-#endif		
+#endif
 
 		if (newRatio < ratio)
 		{
@@ -679,14 +707,21 @@ bool CWrapEngine::WrapRecursive(std::vector<wxWindow*>& windows, double ratio, c
 
 		currentRatio = newRatio;
 	}
-#if WRAPDEBUG > 0		
+#if WRAPDEBUG > 0
 		printf("Performing final wrap with bestwidth %d\n", bestWidth);
-#endif		
-	for (std::vector<wxWindow*>::iterator iter = all_windows.begin(); iter != all_windows.end(); iter++)
+#endif
+#ifdef __WXMAC__
+			const int offset = 6;
+#elif defined(__WXGTK__)
+			const int offset = 0;
+#else
+			const int offset = 0;
+#endif
+	for (std::vector<wxWindow*>::iterator iter = all_windows.begin(); iter != all_windows.end(); ++iter)
 	{
 		wxSizer *pSizer = (*iter)->GetSizer();
 
-		int res = WrapRecursive(*iter, pSizer, bestWidth);
+		int res = WrapRecursive(*iter, pSizer, bestWidth - offset);
 
 		if (res & wrap_didwrap)
 		{
@@ -707,7 +742,6 @@ bool CWrapEngine::WrapRecursive(std::vector<wxWindow*>& windows, double ratio, c
 wxString CWrapEngine::UnwrapText(const wxString& text)
 {
 	wxString unwrapped;
-#if wxUSE_UNICODE
 	int lang = wxGetApp().GetCurrentLanguage();
 	if (lang == wxLANGUAGE_CHINESE || lang == wxLANGUAGE_CHINESE_SIMPLIFIED ||
 		lang == wxLANGUAGE_CHINESE_TRADITIONAL || lang == wxLANGUAGE_CHINESE_HONGKONG ||
@@ -740,9 +774,19 @@ wxString CWrapEngine::UnwrapText(const wxString& text)
 		}
 	}
 	else
-#endif
 	{
 		unwrapped = text;
+
+		// Special handling for unwrapping of URLs
+		int pos;
+		while ( (pos = unwrapped.Find(_T("&&\n"))) > 0 )
+		{
+			if (unwrapped[pos - 1] == ' ')
+				unwrapped = unwrapped.Left(pos + 2) + _T(" ") + unwrapped.Mid(pos + 3);
+			else
+				unwrapped = unwrapped.Left(pos + 2) + unwrapped.Mid(pos + 3);				
+		}
+
 		unwrapped.Replace(_T("\n"), _T(" "));
 		unwrapped.Replace(_T("\r"), _T(""));
 	}
@@ -765,7 +809,7 @@ bool CWrapEngine::UnwrapRecursive(wxWindow* wnd, wxSizer* sizer)
 			if (text)
 			{
 				wxString unwrapped = UnwrapText(text->GetLabel());
-                text->SetLabel(unwrapped);
+				text->SetLabel(unwrapped);
 
 				continue;
 			}
@@ -1121,7 +1165,6 @@ void CWrapEngine::ClearCache()
 
 void CWrapEngine::CheckLanguage()
 {
-#if wxUSE_UNICODE
 	// Just don't bother with wrapping on anything other than UCS-2
 	// FIXME: Use charset conversion routines to convert into UCS-2 and back into
 	//        local charset if not using unicode.
@@ -1136,7 +1179,6 @@ void CWrapEngine::CheckLanguage()
 		m_noWrapChars = noWrapChars_Chinese;
 	}
 	else
-#endif
 	{
 		m_wrapOnEveryChar = false;
 		m_noWrapChars = 0;
