@@ -4,6 +4,8 @@
 #include <wx/ffile.h>
 #include <wx/log.h>
 
+#include <local_filesys.h>
+
 CXmlFile::CXmlFile(const wxString& fileName)
 {
 	m_pPrinter = 0;
@@ -53,15 +55,18 @@ TiXmlElement* CXmlFile::Load(const wxFileName& fileName)
 	m_pDocument = 0;
 
 	wxString error;
-	TiXmlElement* pElement = GetXmlFile(m_fileName, true, &error);
+	TiXmlElement* pElement = GetXmlFile(m_fileName, &error);
 	if (!pElement)
 	{
-		m_error.Printf(_("The file '%s' could not be loaded."), m_fileName.GetFullPath().c_str());
 		if (!error.empty())
-			m_error += _T("\n") + error;
-		else
-			m_error += wxString(_T("\n")) + _("Make sure the file can be accessed and is a well-formed XML document.");
-		m_modificationTime = wxDateTime();
+		{
+			m_error.Printf(_("The file '%s' could not be loaded."), m_fileName.GetFullPath().c_str());
+			if (!error.empty())
+				m_error += _T("\n") + error;
+			else
+				m_error += wxString(_T("\n")) + _("Make sure the file can be accessed and is a well-formed XML document.");
+			m_modificationTime = wxDateTime();
+		}
 		return 0;
 	}
 
@@ -168,49 +173,8 @@ wxString ConvLocal(const char *value)
 	return wxString(wxConvUTF8.cMB2WC(value), *wxConvCurrent);
 }
 
-void AddTextElement(TiXmlElement* node, const char* name, const wxString& value)
+void RemoveTextElement(TiXmlElement* node)
 {
-	wxASSERT(node);
-
-	char* utf8 = ConvUTF8(value);
-	if (!utf8)
-		return;
-
-	TiXmlElement *element = new TiXmlElement(name);
-
-	element->LinkEndChild(new TiXmlText(utf8));
-	delete [] utf8;
-
-	node->LinkEndChild(element);
-}
-
-void AddTextElement(TiXmlElement* node, const char* name, int value)
-{
-	char buffer[sizeof(int) * 8]; // Always big enough
-	sprintf(buffer, "%d", value);
-	AddTextElementRaw(node, name, buffer);
-}
-
-void AddTextElementRaw(TiXmlElement* node, const char* name, const char* value)
-{
-	wxASSERT(node);
-	wxASSERT(value && *value);
-
-	TiXmlElement *element = new TiXmlElement(name);
-
-	element->LinkEndChild(new TiXmlText(value));
-
-	node->LinkEndChild(element);
-}
-
-void AddTextElement(TiXmlElement* node, const wxString& value)
-{
-	wxASSERT(node);
-
-	char* utf8 = ConvUTF8(value);
-	if (!utf8)
-		return;
-
 	for (TiXmlNode* pChild = node->FirstChild(); pChild; pChild = pChild->NextSibling())
 	{
 		if (!pChild->ToText())
@@ -219,8 +183,60 @@ void AddTextElement(TiXmlElement* node, const wxString& value)
 		node->RemoveChild(pChild);
 		break;
 	}
+}
 
-	node->LinkEndChild(new TiXmlText(utf8));
+void AddTextElement(TiXmlElement* node, const char* name, const wxString& value, bool overwrite)
+{
+	wxASSERT(node);
+
+	char* utf8 = ConvUTF8(value);
+	if (!utf8)
+		return;
+
+	AddTextElementRaw(node, name, utf8, overwrite);
+	delete [] utf8;
+}
+
+void AddTextElement(TiXmlElement* node, const char* name, int value, bool overwrite)
+{
+	char buffer[sizeof(int) * 8]; // Always big enough
+	sprintf(buffer, "%d", value);
+	AddTextElementRaw(node, name, buffer, overwrite);
+}
+
+void AddTextElementRaw(TiXmlElement* node, const char* name, const char* value, bool overwrite)
+{
+	wxASSERT(node);
+	wxASSERT(value);
+
+	TiXmlElement *element = 0;
+	if (overwrite)
+	{
+		element = node->FirstChildElement(name);
+		if (element)
+			RemoveTextElement(element);
+	}
+
+	if (!element)
+	{
+		element = new TiXmlElement(name);
+		node->LinkEndChild(element);
+	}
+
+	if (*value)
+		element->LinkEndChild(new TiXmlText(value));
+}
+
+void AddTextElement(TiXmlElement* node, const wxString& value)
+{
+	wxASSERT(node);
+	wxASSERT(value);
+
+	char* utf8 = ConvUTF8(value);
+	if (!utf8)
+		return;
+
+	AddTextElementRaw(node, utf8);
 	delete [] utf8;
 }
 
@@ -234,18 +250,12 @@ void AddTextElement(TiXmlElement* node, int value)
 void AddTextElementRaw(TiXmlElement* node, const char* value)
 {
 	wxASSERT(node);
-	wxASSERT(value && *value);
+	wxASSERT(value);
 
-	for (TiXmlNode* pChild = node->FirstChild(); pChild; pChild = pChild->NextSibling())
-	{
-		if (!pChild->ToText())
-			continue;
+	RemoveTextElement(node);
 
-		node->RemoveChild(pChild);
-		break;
-	}
-
-    node->LinkEndChild(new TiXmlText(value));
+	if (*value)
+		node->LinkEndChild(new TiXmlText(value));
 }
 
 wxString GetTextElement_Trimmed(TiXmlElement* node, const char* name)
@@ -383,7 +393,7 @@ bool GetTextElementBool(TiXmlElement* node, const char* name, bool defValue /*=f
 	const char* str = textNode->Value();
 	if (!str)
 		return defValue;
-	
+
 	switch (str[0])
 	{
 	case '0':
@@ -428,7 +438,7 @@ bool LoadXmlDocument(TiXmlDocument* pXmlDocument, const wxString& file, wxString
 
 // Opens the specified XML file if it exists or creates a new one otherwise.
 // Returns 0 on error.
-TiXmlElement* GetXmlFile(wxFileName file, bool create /*=true*/, wxString* error /*=0*/)
+TiXmlElement* GetXmlFile(wxFileName file, wxString* error /*=0*/)
 {
 	if (wxFileExists(file.GetFullPath()) && file.GetSize() > 0)
 	{
@@ -461,23 +471,13 @@ TiXmlElement* GetXmlFile(wxFileName file, bool create /*=true*/, wxString* error
 	}
 	else
 	{
-		// File does not exist
-		if (!create)
-			return 0;
-		
-		// create new XML document
+		// File does not exist, return empty XML document.
 		TiXmlDocument* pXmlDocument = new TiXmlDocument();
 		pXmlDocument->SetCondenseWhiteSpace(false);
 		pXmlDocument->LinkEndChild(new TiXmlDeclaration("1.0", "UTF-8", "yes"));
 
 		pXmlDocument->LinkEndChild(new TiXmlElement("FileZilla3"));
 
-		if (!SaveXmlFile(file, pXmlDocument, 0))
-		{
-			delete pXmlDocument;
-			return 0;
-		}
-
 		return pXmlDocument->FirstChildElement("FileZilla3");
 	}
 }
@@ -492,8 +492,16 @@ bool SaveXmlFile(const wxFileName& file, TiXmlNode* node, wxString* error /*=0*/
 	TiXmlDocument* pDocument = node->GetDocument();
 
 	bool exists = false;
-	if (wxFileExists(fullPath))
+
+	bool isLink = false;
+	int flags = 0;
+	if (CLocalFileSystem::GetFileInfo( fullPath, isLink, 0, 0, &flags ) == CLocalFileSystem::file)
 	{
+#ifdef __WXMSW__
+		if (flags & FILE_ATTRIBUTE_HIDDEN)
+			SetFileAttributes(fullPath, flags & ~FILE_ATTRIBUTE_HIDDEN);
+#endif
+
 		exists = true;
 		bool res;
 		if (!move)
@@ -664,7 +672,7 @@ void SetServer(TiXmlElement *node, const CServer& server)
 		return;
 
 	bool kiosk_mode = COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) != 0;
-	
+
 	node->Clear();
 
 	AddTextElement(node, "Host", server.GetHost());
@@ -729,7 +737,7 @@ void SetServer(TiXmlElement *node, const CServer& server)
 		if (!postLoginCommands.empty())
 		{
 			TiXmlElement* pElement = node->LinkEndChild(new TiXmlElement("PostLoginCommands"))->ToElement();
-			for (std::vector<wxString>::const_iterator iter = postLoginCommands.begin(); iter != postLoginCommands.end(); iter++)
+			for (std::vector<wxString>::const_iterator iter = postLoginCommands.begin(); iter != postLoginCommands.end(); ++iter)
 				AddTextElement(pElement, "Command", *iter);
 		}
 	}
@@ -748,7 +756,7 @@ void SetTextAttribute(TiXmlElement* node, const char* name, const wxString& valu
 	if (!utf8)
 		return;
 
-    node->SetAttribute(name, utf8);
+	node->SetAttribute(name, utf8);
 	delete [] utf8;
 }
 
