@@ -521,7 +521,7 @@ void CDirectoryListingParserTest::InitEntries()
 				_T("root other"),
 				CDirentry::flag_dir | CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
 				_T(""),
-				wxDateTime(26, wxDateTime::Sep, calcYear(7, 26), 20, 10)
+				wxDateTime(26, wxDateTime::Sep, calcYear(9, 26), 20, 10)
 			},
 			DEFAULT
 		});
@@ -1287,6 +1287,20 @@ void CDirectoryListingParserTest::InitEntries()
 			DEFAULT
 		});
 
+		m_entries.push_back((t_entry){
+			"type=OS.UNIX=symlink; 76b MLSD symlink",
+			{
+				_T("76b MLSD symlink"),
+				-1,
+				_T(""),
+				_T(""),
+				CDirentry::flag_dir | CDirentry::flag_link,
+				_T(""),
+				wxDateTime()
+			},
+			DEFAULT
+		});
+
 		// Old ietf draft for MLST earlier than mlst-07 has no trailing semicolon after facts
 		m_entries.push_back((t_entry){
 			"type=file 77 MLSD file no trailing semicolon after facts < mlst-07",
